@@ -14,6 +14,7 @@ EXTERN unsigned int user_modes[];
 #define FLAGS_CONNECTING    0x00000010UL /* Currently connecting not bursted */
 #define FLAGS_ONACCESS      0x00000020UL /* Client isnt authed with nickserv but does match the access list*/
 #define FLAGS_ENFORCE       0x00000040UL /* User is to be enforced */
+#define FLAGS_SENTCERT      0x00000080UL /* User identified via SSL */
 
 #define STAT_SERVER         0x01
 #define STAT_CLIENT         0x02
@@ -45,23 +46,26 @@ EXTERN unsigned int user_modes[];
 #define UMODE_IDENTIFIED  0x100000 /* Registered with nickserv */
 #define UMODE_ALL    UMODE_SERVNOTICE
 
+#define HasID(x)		((x)->id[0] != '\0')
+
 #define IsConnecting(x)         ((x)->flags & FLAGS_CONNECTING)
 #define IsDefunct(x)            ((x)->flags & (FLAGS_DEADSOCKET|FLAGS_CLOSING))
 #define IsDead(x)               ((x)->flags & FLAGS_DEADSOCKET)
 #define IsClosing(x)            ((x)->flags & FLAGS_CLOSING)
 #define IsOnAccess(x)           ((x)->flags & FLAGS_ONACCESS)
 #define IsEnforce(x)            ((x)->flags & FLAGS_ENFORCE)
-#define IsKilled(x)             ((x)->flags & FLAGS_KILLED)
+#define IsSentCert(x)           ((x)->flags & FLAGS_SENTCERT)
 
 #define SetConnecting(x)        ((x)->flags |= FLAGS_CONNECTING)
 #define SetClosing(x)           ((x)->flags |= FLAGS_CLOSING)
 #define SetOnAccess(x)          ((x)->flags |= FLAGS_ONACCESS)
 #define SetEnforce(x)           ((x)->flags |= FLAGS_ENFORCE)
-#define SetKilled(x)            ((x)->flags |= FLAGS_KILLED)
+#define SetSentCert(x)          ((x)->flags |= FLAGS_SENTCERT)
 
 #define ClearConnecting(x)      ((x)->flags &= ~FLAGS_CONNECTING)
 #define ClearOnAccess(x)        ((x)->flags &= ~FLAGS_ONACCESS)
 #define ClearEnforce(x)         ((x)->flags &= ~FLAGS_ENFORCE)
+#define ClearSentCert(x)        ((x)->flags &= ~FLAGS_SENTCERT)
 
 #define IsServer(x)             ((x)->status & STAT_SERVER)
 #define IsClient(x)             ((x)->status & STAT_CLIENT)
@@ -92,6 +96,8 @@ EXTERN unsigned int user_modes[];
 #define MODE_ADD    1
 #define MODE_DEL   -1
 
+#define SHA1_DIGEST_LENGTH 40
+
 struct Server
 {
   dlink_node node;
@@ -116,7 +122,7 @@ struct Client
   struct Client *from;
   struct Client *servptr;
   struct Client *uplink;        /* services uplink server */
-  struct Client *release_to;    /* The client this one will give its nick to */
+  char *release_to;    /* The name of theclient this one will give its nick to */
 
   struct Nick   *nickname;
 
@@ -128,6 +134,7 @@ struct Client
   char          id[IDLEN + 1];      /* client ID, unique ID per client */
   char          info[REALLEN + 1];  /* Free form additional client info */
   char          username[USERLEN + 1];
+  char          certfp[SHA1_DIGEST_LENGTH+1];
     
   struct Server      *server;
 
