@@ -19,6 +19,14 @@ struct ChanAccess
   unsigned int level;
 };
 
+struct JupeEntry
+{
+  unsigned int id;
+  unsigned int setter;
+  char *name;
+  char *reason;
+};
+
 struct DBResult
 {
   yada_rc_t *rc;
@@ -27,6 +35,8 @@ struct DBResult
 
 struct InfoChanList
 {
+  int channel_id;
+  int ilevel;
   char *channel;
   char *level;
 };
@@ -48,7 +58,9 @@ enum db_list_type
   CHAN_LIST,
   CHAN_LIST_OPER,
   CHAN_FORBID_LIST,
-  EXPIRING_AKILL_LIST
+  EXPIRING_AKILL_LIST,
+  CERT_LIST,
+  JUPES_LIST
 };
 
 enum db_queries
@@ -148,15 +160,27 @@ enum db_queries
   GET_CHANNELS_OPER,
   GET_CHANNEL_FORBID_LIST,
   SAVE_NICK,
+  INSERT_NICKCERT,
+  GET_NICKCERT,
+  DELETE_NICKCERT,
+  DELETE_NICKCERT_IDX,
+  DELETE_ALL_NICKCERT,
+  INSERT_JUPE,
+  GET_JUPES,
+  DELETE_JUPES_NAME,
+  FIND_JUPE,
+  COUNT_CHANNEL_ACCESS_LIST,
   QUERY_COUNT
 };
 
-enum query_types {
+enum query_types 
+{
   QUERY,
   EXECUTE
 };
 
-typedef struct query {
+typedef struct query 
+{
   int index;
   const char *name;
   yada_rc_t *rc;
@@ -208,6 +232,7 @@ struct Nick *db_find_nick(const char *);
 struct RegChannel *db_find_chan(const char *);
 struct ServiceBan *db_find_akill(const char *);
 struct ChanAccess *db_find_chanaccess(unsigned int, unsigned int);
+struct JupeEntry  *db_find_jupe(const char *);
 
 int   db_list_add(unsigned int, const void *);
 void *db_list_first(unsigned int, unsigned int, void **);
@@ -217,10 +242,13 @@ int   db_list_del(unsigned int, unsigned int, const char *);
 int   db_list_del_index(unsigned int, unsigned int, unsigned int);
 
 int db_get_num_masters(unsigned int);
+int db_get_num_channel_accesslist_entries(unsigned int);
 
 int db_add_sentmail(unsigned int, const char *);
 int db_is_mailsent(unsigned int, const char *);
 
 int db_save_nick(struct Nick *);
 
+void db_reopen_log();
+
 #endif /* INCLUDED_dbm_h */
