@@ -20,7 +20,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: hook.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: hook.c 1055 2007-05-24 22:21:32Z swalsh $
  */
 
 #include "libioinc.h"
@@ -204,7 +204,9 @@ install_hook(struct Callback *cb, CBFUNC *hook)
 void
 uninstall_hook(struct Callback *cb, CBFUNC *hook)
 {
-  /* let it core if not found */
+  if(cb == NULL)
+    return;
+
   dlink_node *ptr = dlinkFind(&cb->chain, hook);
 
   dlinkDelete(ptr, &cb->chain);
