@@ -19,12 +19,13 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: chanserv.c 1061 2007-06-02 16:40:56Z tjfontaine $
+ *  $Id: chanserv.c 1140 2007-08-20 15:43:40Z swalsh $
  */
 
 #include "stdinc.h"
 
 static struct Service *chanserv = NULL;
+static struct Client *chanserv_client = NULL;
 
 static dlink_node *cs_cmode_hook;
 static dlink_node *cs_join_hook;
@@ -78,8 +79,10 @@ static void m_akick_del(struct Service *, struct Client *, int, char *[]);
 static void m_akick_enforce(struct Service *, struct Client *, int, char *[]);
 
 static void m_clear_bans(struct Service *, struct Client *, int, char *[]);
+static void m_clear_quiets(struct Service *, struct Client *, int, char *[]);
 static void m_clear_ops(struct Service *, struct Client *, int, char *[]);
 static void m_clear_voices(struct Service *, struct Client *, int, char *[]);
+static void m_clear_modes(struct Service *, struct Client *, int, char *[]);
 static void m_clear_users(struct Service *, struct Client *, int, char *[]);
 
 static void m_op(struct Service *, struct Client *, int, char *[]);
@@ -88,6 +91,7 @@ static void m_voice(struct Service *, struct Client *, int, char *[]);
 static void m_devoice(struct Service *, struct Client *, int, char *[]);
 static void m_invite(struct Service *, struct Client *, int, char *[]);
 static void m_unban(struct Service *, struct Client *, int, char *[]);
+static void m_unquiet(struct Service *, struct Client *, int, char *[]);
 static void m_list(struct Service *, struct Client *, int, char *[]);
 
 static void m_access_add(struct Service *, struct Client *, int, char *[]);
@@ -98,6 +102,7 @@ static void m_access_list(struct Service *, struct Client *, int, char *[]);
 static int m_set_flag(struct Service *, struct Client *, char *, char *, int, char *);
 static int m_set_string(struct Service *, struct Client *, const char *, 
     const char *, unsigned int, const char *, char **, int);
+static void m_delete_autolimit(struct Channel *chptr);
 
 static struct ServiceMessage register_msgtab = {
   NULL, "REGISTER", 0, 2, 2, SFLG_UNREGOK|SFLG_NOMAXPARAM, CHIDENTIFIED_FLAG, 
@@ -218,6 +223,11 @@ static struct ServiceMessage unban_msgtab = {
   CS_HELP_UNBAN_SHORT, CS_HELP_UNBAN_LONG, m_unban
 };
 
+static struct ServiceMessage unquiet_msgtab = {
+  NULL, "UNQUIET", 0, 1, 1, SFLG_CHANARG, MEMBER_FLAG,
+  CS_HELP_UNQUIET_SHORT, CS_HELP_UNQUIET_LONG, m_unquiet
+};
+
 static struct ServiceMessage invite_msgtab = {
   NULL, "INVITE", 0, 1, 2, SFLG_CHANARG, MEMBER_FLAG,
   CS_HELP_INVITE_SHORT, CS_HELP_INVITE_LONG, m_invite
@@ -226,10 +236,14 @@ static struct ServiceMessage invite_msgtab = {
 static struct ServiceMessage clear_sub[] = {
   { NULL, "BANS", 0, 1, 1, SFLG_KEEPARG|SFLG_CHANARG, CHANOP_FLAG, 
     CS_HELP_CLEAR_BANS_SHORT, CS_HELP_CLEAR_BANS_LONG, m_clear_bans },
+  { NULL, "QUIETS", 0, 1, 1, SFLG_KEEPARG|SFLG_CHANARG, CHANOP_FLAG,
+    CS_HELP_CLEAR_QUIETS_SHORT, CS_HELP_CLEAR_QUIETS_LONG, m_clear_quiets },
   { NULL, "OPS", 0, 1, 1, SFLG_KEEPARG|SFLG_CHANARG, CHANOP_FLAG, 
     CS_HELP_CLEAR_OPS_SHORT, CS_HELP_CLEAR_OPS_LONG, m_clear_ops },
   { NULL, "VOICES", 0, 1, 1, SFLG_KEEPARG|SFLG_CHANARG, CHANOP_FLAG, 
     CS_HELP_CLEAR_VOICES_SHORT, CS_HELP_CLEAR_VOICES_LONG, m_clear_voices },
+  { NULL, "MODES", 0, 1, 1, SFLG_KEEPARG|SFLG_CHANARG, CHANOP_FLAG,
+    CS_HELP_CLEAR_MODES_SHORT, CS_HELP_CLEAR_MODES_LONG, m_clear_modes },
   { NULL, "USERS", 0, 1, 1, SFLG_KEEPARG|SFLG_CHANARG, MASTER_FLAG, 
     CS_HELP_CLEAR_USERS_SHORT, CS_HELP_CLEAR_USERS_LONG, m_clear_users },
   { NULL, NULL, 0, 0, 0, 0, 0, 0, 0, NULL }
@@ -260,13 +274,13 @@ static struct ServiceMessage unforbid_msgtab = {
   CS_HELP_UNFORBID_SHORT, CS_HELP_UNFORBID_LONG, m_unforbid
 };
 
-INIT_MODULE(chanserv, "$Revision: 1061 $")
+INIT_MODULE(chanserv, "$Revision: 1140 $")
 {
   chanserv = make_service("ChanServ");
   clear_serv_tree_parse(&chanserv->msg_tree);
   dlinkAdd(chanserv, &chanserv->node, &services_list);
   hash_add_service(chanserv);
-  introduce_client(chanserv->name);
+  chanserv_client = introduce_client(chanserv->name, chanserv->name, TRUE);
   load_language(chanserv->languages, "chanserv.en");
 /*  load_language(chanserv, "chanserv.rude");
   load_language(chanserv, "chanserv.de");
@@ -285,6 +299,7 @@ INIT_MODULE(chanserv, "$Revision: 1061 $")
   mod_add_servcmd(&chanserv->msg_tree, &invite_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &clear_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &unban_msgtab);
+  mod_add_servcmd(&chanserv->msg_tree, &unquiet_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &invite_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &sudo_msgtab);
   mod_add_servcmd(&chanserv->msg_tree, &list_msgtab);
@@ -318,7 +333,7 @@ CLEANUP_MODULE
 
   unload_languages(chanserv->languages);
 
-  exit_client(find_client(chanserv->name), &me, "Service unloaded");
+  exit_client(chanserv_client, &me, "Service unloaded");
   hash_del_service(chanserv);
   dlinkDelete(&chanserv->node, &services_list);
   eventDelete(process_limit_list, NULL);
@@ -362,14 +377,35 @@ process_expireban_list(void *param)
     {
       struct Ban *banptr = bptr->data;
       char ban[IRC_BUFSIZE+1];
+      time_t delta = CurrentTime - banptr->when;
+      time_t maxtime = 1*60*60;
 
-      if((CurrentTime - banptr->when) > (1*60*60))
+      if(delta > maxtime)
       {
         snprintf(ban, IRC_BUFSIZE, "%s!%s@%s", banptr->name, banptr->username,
             banptr->host);
+        ilog(L_DEBUG, "ChanServ ExpireBan: BAN %s %d %s", chptr->chname,
+          (int)delta, ban);
         unban_mask(chanserv, chptr, ban);
       }
     }
+
+    DLINK_FOREACH_SAFE(bptr, bnptr, chptr->quietlist.head)
+    {
+      struct Ban *banptr = bptr->data;
+      char ban[IRC_BUFSIZE+1];
+      time_t delta = CurrentTime - banptr->when;
+      time_t maxtime = 1*60*60;
+
+      if(delta > maxtime)
+      {
+        snprintf(ban, IRC_BUFSIZE, "%s!%s@%s", banptr->name, banptr->username,
+            banptr->host);
+        ilog(L_DEBUG, "ChanServ ExpireBan: QUIET %s %d %s", chptr->chname,
+          (int)delta, ban);
+        unquiet_mask(chanserv, chptr, ban);
+      }
+    }
   }
 }
 
@@ -379,6 +415,7 @@ m_register(struct Service *service, struct Client *client,
 {
   struct Channel    *chptr;
   struct RegChannel *regchptr;
+  char desc[IRC_BUFSIZE+1];
 
   assert(parv[1]);
 
@@ -431,7 +468,8 @@ m_register(struct Service *service, struct Client *client,
 
   regchptr = MyMalloc(sizeof(struct RegChannel));
   strlcpy(regchptr->channel, parv[1], sizeof(regchptr->channel));
-  DupString(regchptr->description, parv[2]);
+  join_params(desc, parc-1, &parv[2]);
+  DupString(regchptr->description, desc);
 
   if (db_register_chan(regchptr, client->nickname->id))
   {
@@ -568,6 +606,9 @@ m_info(struct Service *service, struct Client *client,
   reply_user(service, service, client, CS_INFO_OPTION, "LEAVEOPS",
       regchptr->leaveops ? "ON" : "OFF");
 
+  reply_user(service, service, client, CS_INFO_OPTION, "FLOODSERV",
+      regchptr->floodserv ? "ON" : "OFF");
+
   if(access != NULL)
     MyFree(access);
 
@@ -650,6 +691,9 @@ m_access_add(struct Service *service, struct Client *client,
         level_added);
     ilog(L_DEBUG, "%s (%s@%s) added AE %s(%d) to %s", client->name, 
         client->username, client->host, parv[2], access->level, parv[1]);
+    send_chops_notice(service, chptr, "[%s ChanOps] %s Added %s to %s "
+        "access list as %s", chptr->chname, client->name, parv[2], 
+        chptr->chname, level_added); 
   }
   else
     reply_user(service, service, client, CS_ACCESS_ADDFAIL, parv[2], parv[1],
@@ -708,6 +752,8 @@ m_access_del(struct Service *service, struct Client *client,
     reply_user(service, service, client, CS_ACCESS_DELOK, parv[2], parv[1]);
     ilog(L_DEBUG, "%s (%s@%s) removed AE %s from %s", 
       client->name, client->username, client->host, parv[2], parv[1]);
+    send_chops_notice(service, chptr, "[%s ChanOps] %s Removed %s from %s "
+        "access list", chptr->chname, client->name, parv[2], chptr->chname); 
   }
   else
     reply_user(service, service, client, CS_ACCESS_DELFAIL, parv[2], parv[1]);
@@ -899,6 +945,7 @@ m_set_mlock(struct Service *service, struct Client *client, int parc,
   struct Channel *chptr;
   struct RegChannel *regchptr;
   char value[IRC_BUFSIZE+1];
+  char limit_cache;
 
   chptr = hash_find_channel(parv[1]);
   regchptr = chptr == NULL ? db_find_chan(parv[1]) : chptr->regchan;
@@ -914,15 +961,27 @@ m_set_mlock(struct Service *service, struct Client *client, int parc,
     return;
   }
 
+  limit_cache = regchptr->autolimit;
   if(set_mode_lock(service, parv[1], client, value, &regchptr->mlock))
   {
+    if(limit_cache != regchptr->autolimit)
+    {
+      m_set_flag(service, client, parv[1], "OFF", SET_CHAN_AUTOLIMIT, "AUTOLIMIT");
+      if(chptr != NULL)
+        m_delete_autolimit(chptr);
+      reply_user(service, service, client, CS_MLOCK_CONFLICT_LIMIT);
+    }
+
     reply_user(service, service, client, CS_SET_SUCCESS, "MLOCK",
         regchptr->mlock == NULL ? "Not set" : regchptr->mlock, 
         regchptr->channel);
   }
   else
+  {
     reply_user(service, service, client, CS_SET_FAILED, "MLOCK", 
         value, parv[1]);
+    regchptr->autolimit = limit_cache;
+  }
 
   if(chptr == NULL)
     free_regchan(regchptr);
@@ -986,6 +1045,7 @@ m_set_autolimit(struct Service *service, struct Client *client,
 {
   struct Channel *chptr = hash_find_channel(parv[1]);
 
+  /*TODO XXX FIXME Setting AUTOLIMIT and having -l/+l MLOCK will fight each other*/
   m_set_flag(service, client, parv[1], parv[2], SET_CHAN_AUTOLIMIT, "AUTOLIMIT");
   if(chptr != NULL && chptr->regchan != NULL && chptr->regchan->autolimit)
   {
@@ -994,12 +1054,18 @@ m_set_autolimit(struct Service *service, struct Client *client,
   }
   else if(chptr != NULL && chptr->regchan != NULL)
   {
-    dlink_node *ptr;
-
-    if((ptr = dlinkFindDelete(&channel_limit_list, chptr)) != NULL)
-      free_dlink_node(ptr);
+    m_delete_autolimit(chptr);
   }
 }
+
+static void
+m_delete_autolimit(struct Channel *chptr)
+{
+  dlink_node *ptr;
+
+  if((ptr = dlinkFindDelete(&channel_limit_list, chptr)) != NULL)
+    free_dlink_node(ptr);
+}
   
 static void
 m_set_expirebans(struct Service *service, struct Client *client,
@@ -1099,7 +1165,7 @@ m_akick_add(struct Service *service, struct Client *client, int parc,
   }
 
   akick->setter = client->nickname->id;
-  DupString(akick->channel, parv[1]);
+  akick->channel = regchptr->id;
   akick->time_set = CurrentTime;
   akick->duration = 0;
 
@@ -1261,6 +1327,39 @@ m_clear_bans(struct Service *service, struct Client *client, int parc,
       regchptr->channel);
 }
 
+static void
+m_clear_quiets(struct Service *service, struct Client *client, int parc,
+    char *parv[])
+{
+  struct Channel *chptr;
+  struct RegChannel *regchptr;
+  dlink_node *ptr, *nptr;
+  int numbans = 0;
+
+  chptr = hash_find_channel(parv[1]);
+
+  if(chptr == NULL)
+  {
+    reply_user(service, service, client, CS_CHAN_NOT_USED, parv[1]);
+    return;
+  }
+  regchptr = chptr->regchan;
+
+  DLINK_FOREACH_SAFE(ptr, nptr, chptr->quietlist.head)
+  {
+    const struct Ban *banptr = ptr->data;
+    char ban[IRC_BUFSIZE+1];
+
+    snprintf(ban, IRC_BUFSIZE, "%s!%s@%s", banptr->name, banptr->username,
+        banptr->host);
+    unquiet_mask(service, chptr, ban);
+    numbans++;
+  }
+
+  reply_user(service, service, client, CS_CLEAR_QUIETS, numbans,
+      regchptr->channel);
+}
+
 static void
 m_clear_ops(struct Service *service, struct Client *client, int parc, 
     char *parv[])
@@ -1327,6 +1426,52 @@ m_clear_voices(struct Service *service, struct Client *client, int parc,
       regchptr->channel);
 }
 
+static void
+m_clear_modes(struct Service *service, struct Client *client, int parc,
+    char *parv[])
+{
+  struct Channel *chptr;
+  char mbuf[MODEBUFLEN+1];
+  char buf[MODEBUFLEN+1] = "-";
+
+  chptr = hash_find_channel(parv[1]);
+
+  if(chptr == NULL)
+  {
+    reply_user(service, service, client, CS_CHAN_NOT_USED, parv[1]);
+    return;
+  }
+
+  get_modestring(chptr->mode.mode, mbuf, MODEBUFLEN);
+
+  ilog(L_DEBUG, "ChanServ CLEAR MODES: %s has %s", chptr->chname, mbuf);
+
+  strlcat(buf, mbuf, MODEBUFLEN);
+
+  if(chptr->mode.key[0] != '\0')
+  {
+    strlcat(buf, "k", MODEBUFLEN);
+    chptr->mode.key[0] = '\0';
+  }
+
+  if(chptr->mode.limit > 0)
+  {
+    strlcat(buf, "l", MODEBUFLEN);
+    chptr->mode.limit = 0;
+  }
+
+  ilog(L_DEBUG, "ChanServ CLEAR MODES: %s removing %s", chptr->chname, buf);
+
+  send_cmode(service, chptr, buf, "");
+  chptr->mode.mode &= ~chptr->mode.mode;
+
+  ilog(L_DEBUG, "ChanServ CLEAR MODES: %s setting MLOCK %s", chptr->chname,
+    chptr->regchan->mlock);
+  set_mode_lock(service, chptr->chname, NULL, chptr->regchan->mlock, NULL);
+
+  reply_user(service, service, client, CS_CLEAR_MODES, chptr->chname);
+}
+
 static void
 m_clear_users(struct Service *service, struct Client *client, int parc, 
     char *parv[])
@@ -1394,6 +1539,8 @@ m_op(struct Service *service, struct Client *client, int parc, char *parv[])
   {
     op_user(service, chptr, target);
     reply_user(service, service, client, CS_OP, target->name, parv[1]);
+    send_chops_notice(service, chptr, "[%s ChanOps] %s OP %s", 
+        chptr->chname, client->name, target->name, chptr->chname);
   }
   else
     reply_user(service, service, client, CS_ALREADY_OP, target->name, parv[1]);
@@ -1431,6 +1578,8 @@ m_deop(struct Service *service, struct Client *client, int parc, char *parv[])
   {
     deop_user(service, chptr, target);
     reply_user(service, service, client, CS_DEOP, target->name, parv[1]);
+    send_chops_notice(service, chptr, "[%s ChanOps] %s DEOP %s", 
+        chptr->chname, client->name, target->name, chptr->chname);
   }
   else
     reply_user(service, service, client, CS_NOT_OP, target->name, parv[1]);
@@ -1479,6 +1628,8 @@ m_voice(struct Service *service, struct Client *client, int parc, char *parv[])
   {
     voice_user(service, chptr, target);
     reply_user(service, service, client, CS_VOICE, target->name, parv[1]);
+    send_chops_notice(service, chptr, "[%s ChanOps] %s VOICE %s", 
+        chptr->chname, client->name, target->name, chptr->chname); 
   }
   else
     reply_user(service, service, client, CS_ALREADY_VOICE, target->name, parv[1]);
@@ -1527,6 +1678,8 @@ m_devoice(struct Service *service, struct Client *client, int parc, char *parv[]
   {
     devoice_user(service, chptr, target);
     reply_user(service, service, client, CS_DEVOICE, target->name, parv[1]);
+    send_chops_notice(service, chptr, "[%s ChanOps] %s DEVOICE %s", 
+        chptr->chname, client->name, target->name, chptr->chname); 
   }
   else
     reply_user(service, service, client, CS_NOT_VOICE, target->name, parv[1]);
@@ -1569,6 +1722,8 @@ m_invite(struct Service *service, struct Client *client, int parc, char *parv[])
 
   invite_user(service, chptr, target);
   reply_user(service, service, client, CS_INVITED, target->name, parv[1]);
+  send_chops_notice(service, chptr, "[%s ChanOps] %s INVITE %s", 
+      chptr->chname, client->name, target->name, chptr->chname); 
 }
 
 static void
@@ -1595,7 +1750,10 @@ m_unban(struct Service *service, struct Client *client, int parc, char *parv[])
 
     snprintf(ban, IRC_BUFSIZE, "%s!%s@%s", banp->name, banp->username,
         banp->host);
+    send_chops_notice(service, chptr, "[%s ChanOps] %s UNBAN %s!%s@%s", 
+        chptr->chname, client->name, banp->name, banp->username, banp->host);
     unban_mask(service, chptr, ban);
+ 
     numbans++;
 
     banp = find_bmask(client, &chptr->banlist);
@@ -1605,6 +1763,40 @@ m_unban(struct Service *service, struct Client *client, int parc, char *parv[])
       regchptr->channel);
 }
 
+static void
+m_unquiet(struct Service *service, struct Client *client, int parc, char *parv[])
+{
+  struct Channel *chptr;
+  struct RegChannel *regchptr;
+  struct Ban *banp;
+  int numbans = 0;
+
+  chptr = hash_find_channel(parv[1]);
+
+  if(chptr == NULL)
+  {
+    reply_user(service, service, client, CS_CHAN_NOT_USED, parv[1]);
+    return;
+  }
+  regchptr = chptr->regchan;
+
+  banp = find_bmask(client, &chptr->quietlist);
+  while(banp != NULL)
+  {
+    char ban[IRC_BUFSIZE+1];
+
+    snprintf(ban, IRC_BUFSIZE, "%s!%s@%s", banp->name, banp->username,
+        banp->host);
+    unquiet_mask(service, chptr, ban);
+    numbans++;
+
+    banp = find_bmask(client, &chptr->quietlist);
+  }
+
+  reply_user(service, service, client, CS_CLEAR_QUIETS, numbans,
+      regchptr->channel);
+}
+
 static int
 m_set_string(struct Service *service, struct Client *client, 
     const char *channel, const char *field, unsigned int query, 
@@ -1644,6 +1836,9 @@ m_set_string(struct Service *service, struct Client *client,
       ilog(L_DEBUG, "%s (%s@%s) changed %s of %s to %s", 
           client->name, client->username, client->host, field, regchptr->channel, 
           value);
+      send_chops_notice(service, chptr, "[%s ChanOps] %s SET %s to %s", 
+          chptr->chname, client->name, field,
+          value == NULL ? "Not set" : value); 
     }
 
     if(value != NULL)
@@ -2144,11 +2339,11 @@ cs_on_topic_change(va_list args)
       if(chan->topic == NULL || 
           ircncmp(chan->topic, regchptr->topic, TOPICLEN) != 0)
       {
-        send_topic(chanserv, chan, find_client(chanserv->name), regchptr->topic); 
+        send_topic(chanserv, chan, chanserv_client, regchptr->topic); 
       }
     }
     else
-      send_topic(chanserv, chan, find_client(chanserv->name), NULL);
+      send_topic(chanserv, chan, chanserv_client, NULL);
   }
   else
   {
@@ -2165,7 +2360,7 @@ static void *
 cs_on_burst_done(va_list args)
 {
   dlink_node *ptr;
-  struct Client *chanserv_client = find_client(chanserv->name);
+  struct Client *chanserv_client = chanserv_client;
 
   DLINK_FOREACH(ptr, global_channel_list.head)
   {
