@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: irc.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: irc.c 1182 2007-10-26 17:07:49Z swalsh $
  */
 
 #include "stdinc.h"
@@ -38,6 +38,7 @@ static void m_topic(struct Client *, struct Client *, int, char*[]);
 static void m_kill(struct Client *, struct Client *, int, char*[]);
 static void m_kick(struct Client *, struct Client *, int, char*[]);
 static void m_version(struct Client *, struct Client *, int, char*[]);
+static void m_stats(struct Client *, struct Client *, int, char*[]);
 static void m_tburst(struct Client *, struct Client *, int, char*[]);
 
 //static void do_user_modes(struct Client *client, const char *modes);
@@ -58,6 +59,7 @@ static void *irc_sendmsg_server(va_list);
 static void *irc_sendmsg_join(va_list);
 static void *irc_server_connected(va_list);
 static void *irc_sendmsg_nosuchsrv(va_list);
+static void *irc_sendmsg_squit(va_list);
 static void *part_one_client(va_list);
 static char modebuf[MODEBUFLEN];
 static char parabuf[MODEBUFLEN];
@@ -88,7 +90,7 @@ static struct Message trace_msgtab = {
 
 static struct Message stats_msgtab = {
   "STATS", 0, 0, 0, 0, 0, 0,
-  { m_ignore, m_ignore }
+  { m_stats, m_stats }
 };
 
 static struct Message whois_msgtab = {
@@ -213,6 +215,7 @@ static dlink_node *newserver_hook;
 static dlink_node *join_hook;
 static dlink_node *part_hook;
 static dlink_node *nosuchsrv_hook;
+static dlink_node *squit_hook;
 
 struct ModeList ModeList[] = {
   { MODE_NOPRIVMSGS,  'n' },
@@ -224,7 +227,7 @@ struct ModeList ModeList[] = {
   { 0, '\0' }
 };
 
-INIT_MODULE(irc, "$Revision: 1043 $")
+INIT_MODULE(irc, "$Revision: 1182 $")
 {
   connected_hook  = install_hook(connected_cb, irc_server_connected);
   newuser_hook    = install_hook(send_newuser_cb, irc_sendmsg_nick);
@@ -243,6 +246,7 @@ INIT_MODULE(irc, "$Revision: 1043 $")
   join_hook       = install_hook(send_join_cb, irc_sendmsg_join);
   part_hook       = install_hook(send_part_cb, part_one_client);
   nosuchsrv_hook  = install_hook(send_nosuchsrv_cb, irc_sendmsg_nosuchsrv);
+  squit_hook      = install_hook(send_squit_cb, irc_sendmsg_squit);
   mod_add_cmd(&ping_msgtab);
   mod_add_cmd(&server_msgtab);
   mod_add_cmd(&nick_msgtab);
@@ -337,7 +341,7 @@ irc_sendmsg_nick(va_list args)
   char          *umode  = va_arg(args, char *);
   
   // NICK who hop ts umode user host server info
-  sendto_server(client, "NICK %s 1 666 +%saoS %s %s %s :%s", 
+  sendto_server(client, "NICK %s 1 666 +%s %s %s %s :%s", 
     nick, umode, user, host, me.name, info);
 
   return NULL;
@@ -455,9 +459,11 @@ irc_sendmsg_unakill(va_list args)
   char name[NICKLEN];
   char user[USERLEN + 1];
   char host[HOSTLEN + 1];
+  char banmask[IRC_BUFSIZE+1];
   struct split_nuh_item nuh;
 
-  nuh.nuhmask  = mask;
+  ircsprintf(banmask, "%s", mask);
+  nuh.nuhmask = banmask;
   nuh.nickptr  = name;
   nuh.userptr  = user;
   nuh.hostptr  = host;
@@ -487,13 +493,13 @@ irc_sendmsg_topic(va_list args)
 
   if(topic == NULL)
   {
-    sendto_server(uplink, ":%s TBURST 1 %s %lu %s", me.name, chptr->chname,
+    sendto_server(uplink, ":%s TOPIC %s :", source->name, chptr->chname,
         CurrentTime, setter->name);
   }
   else
   {
-    sendto_server(uplink, ":%s TBURST 1 %s %lu %s :%s", me.name, chptr->chname,
-        CurrentTime, setter->name, topic);
+    sendto_server(uplink, ":%s TOPIC %s :%s", source->name, chptr->chname,
+        topic);
   }
 
   return NULL;
@@ -577,6 +583,7 @@ irc_sendmsg_quit(struct Client *client, char *nick, char *reason)
 {
   sendto_server(client, ":%s QUIT :%s", nick, reason);
 }
+#endif
 
 /** SQuit a Server
  * @param
@@ -584,12 +591,17 @@ irc_sendmsg_quit(struct Client *client, char *nick, char *reason)
  * target Server to be squit
  * reason Why the squit?
  */
-static void
-irc_sendmsg_squit(struct Client *client, char *source, char *target, char *reason)
+static void *
+irc_sendmsg_squit(va_list args)
 {
-  sendto_server(client, ":%s SQUIT %s :%s", source, target, reason);
+  struct Client *source = va_arg(args, struct Client *);
+  const char *target    = va_arg(args, const char *);
+  const char *reason    = va_arg(args, const char *);
+
+  sendto_server(me.uplink, ":%s SQUIT %s :%s", source->name, target, reason);
+
+  return NULL;
 }
-#endif
 
 /** Send a PING to a remote client
  * @param
@@ -664,7 +676,15 @@ irc_server_connected(va_list args)
   {
     struct Service *service = ptr->data;
 
-    introduce_client(service->name);
+    introduce_client(service->name, service->name, TRUE);
+  }
+
+  ptr = NULL;
+
+  DLINK_FOREACH(ptr, me.server_list.head)
+  {
+    struct Client *server = ptr->data;
+    introduce_server(server->name, server->info);
   }
 
   return NULL;
@@ -736,8 +756,6 @@ part_one_client(va_list args)
     return NULL;
   }
 
-  remove_user_from_channel(ms);
-
   if(MyConnect(source))
   {
     if (reason[0] != '\0')
@@ -748,6 +766,8 @@ part_one_client(va_list args)
     execute_callback(on_part_cb, client, source, chptr, reason);
   }
 
+  remove_user_from_channel(ms);
+
   return NULL;
 }
 
@@ -996,6 +1016,12 @@ nextnick:
    * - Dianora
    */
 
+  /* There is a slight possibility the join callback above gets rid of the
+   * channel, make sure it still exists */
+
+  if((chptr = hash_find_channel(parv[2])) == NULL)
+    return;
+
   if(isnew)
     execute_callback(on_channel_created_cb, chptr);
 
@@ -1125,8 +1151,11 @@ m_part(struct Client *client, struct Client *source, int parc, char *parv[])
 
   while (name)
   {
-    execute_callback(on_part_cb, client, source, hash_find_channel(name), 
-        reason);
+    struct Channel *chan;
+
+    chan = hash_find_channel(name);
+    if(chan != NULL)
+      execute_callback(on_part_cb, client, source, chan, reason);
     execute_callback(send_part_cb, client, source, name, reason);
     name = strtoken(&p, NULL, ",");
   }
@@ -1197,7 +1226,6 @@ m_squit(struct Client *client, struct Client *source, int parc, char *parv[])
     comment[REASONLEN] = '\0';
 
   exit_client(target, source, comment);
-//  chain_squit(client, source, comment);
 
   if(target == me.uplink)
     me.uplink = NULL;
@@ -1303,6 +1331,52 @@ m_version(struct Client *client_p, struct Client *source_p,
   execute_callback(send_nosuchsrv_cb, source_p->name, parv[1]);
 }
 
+static void
+m_stats(struct Client *client_p, struct Client *source_p,
+  int parc, char *parv[])
+{
+  struct Client *target = NULL;
+
+  if((target = find_server(parv[2])) == NULL)
+    return;
+
+
+  if(IsMe(target) && *parv[1] == 'z' && IsOper(source_p))
+  {
+    ilog(L_NOTICE, "STATS %c requested by %s (%s@%s) [%s]",
+      *parv[1], source_p->name, source_p->username,
+      source_p->host, source_p->servptr->name);
+
+    dlink_list *usage = block_heap_get_usage();
+    dlink_node *ptr = NULL, *next_ptr = NULL;
+    struct BlockHeapInfo *bi = NULL;
+
+    DLINK_FOREACH_SAFE(ptr, next_ptr, usage->head)
+    {
+      bi = (struct BlockHeapInfo *)ptr->data;
+      sendto_server(me.uplink,
+        ":%s %d %s z :%s mempool: used %u/%u free %u/%u (size %u/%u)",
+        me.name, 249, source_p->name, bi->name, bi->used_elm, bi->used_mem,
+        bi->free_elm, bi->free_mem, bi->size_elm, bi->size_mem);
+      dlinkDelete(ptr, usage);
+      MyFree(bi->name);
+      bi->name = NULL;
+      MyFree(bi);
+    }
+    MyFree(usage);
+    sendto_server(me.uplink, ":%s 219 %s %c :End of /STATS report",
+      me.name, source_p->name, 'z');
+  }
+  else if(!IsMe(target))
+  {
+    execute_callback(send_nosuchsrv_cb, source_p->name, parv[2]);
+  }
+  else
+  {
+    /* ERR NO PERM */
+  }
+}
+
 static void
 m_tburst(struct Client *client_p, struct Client *source_p, int parc, 
     char *parv[])
