@@ -23,17 +23,19 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: nickserv.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: nickserv.c 1184 2007-10-30 23:11:26Z swalsh $
  */
 
 #include "stdinc.h"
 
 static struct Service *nickserv = NULL;
+static struct Client *nickserv_client = NULL;
 
 static dlink_node *ns_umode_hook;
 static dlink_node *ns_nick_hook;
 static dlink_node *ns_newuser_hook;
 static dlink_node *ns_quit_hook;
+static dlink_node *ns_certfp_hook;
 
 static dlink_list nick_enforce_list = { NULL, NULL, 0 };
 static dlink_list nick_release_list = { NULL, NULL, 0 };
@@ -47,6 +49,7 @@ static void *ns_on_umode_change(va_list);
 static void *ns_on_newuser(va_list);
 static void *ns_on_nick_change(va_list);
 static void *ns_on_quit(va_list);
+static void *ns_on_certfp(va_list);
 static void m_drop(struct Service *, struct Client *, int, char *[]);
 static void m_help(struct Service *, struct Client *, int, char *[]);
 static void m_identify(struct Service *, struct Client *, int, char *[]);
@@ -60,6 +63,7 @@ static void m_regain(struct Service *,struct Client *, int, char *[]);
 static void m_sudo(struct Service *, struct Client *, int, char *[]);
 static void m_sendpass(struct Service *, struct Client *, int, char *[]);
 static void m_list(struct Service *, struct Client *, int, char *[]);
+static void m_status(struct Service *, struct Client *, int, char*[]);
 
 static void m_set_language(struct Service *, struct Client *, int, char *[]);
 static void m_set_password(struct Service *, struct Client *, int, char *[]);
@@ -76,6 +80,10 @@ static void m_access_add(struct Service *, struct Client *, int, char *[]);
 static void m_access_list(struct Service *, struct Client *, int, char *[]);
 static void m_access_del(struct Service *, struct Client *, int, char *[]);
 
+static void m_cert_add(struct Service *, struct Client *, int, char *[]);
+static void m_cert_list(struct Service *, struct Client *, int, char *[]);
+static void m_cert_del(struct Service *, struct Client *, int, char *[]);
+
 static struct ServiceMessage register_msgtab = {
   NULL, "REGISTER", 0, 2, 2, 0, USER_FLAG, NS_HELP_REG_SHORT, NS_HELP_REG_LONG, 
   m_register 
@@ -133,6 +141,21 @@ static struct ServiceMessage cloakstring_msgtab = {
     NS_HELP_CLOAKSTRING_LONG, m_cloakstring 
 }; 
 
+static struct ServiceMessage cert_sub[] = {
+  { NULL, "ADD", 0, 0, 1, 0, IDENTIFIED_FLAG, NS_HELP_CERT_ADD_SHORT, 
+    NS_HELP_CERT_ADD_LONG, m_cert_add },
+  { NULL, "LIST", 0, 0, 0, 0, IDENTIFIED_FLAG, NS_HELP_CERT_LIST_SHORT, 
+    NS_HELP_CERT_LIST_LONG, m_cert_list },
+  { NULL, "DEL", 0, 1, 1, 0, IDENTIFIED_FLAG, NS_HELP_CERT_DEL_SHORT, 
+    NS_HELP_CERT_DEL_LONG, m_cert_del },
+  { NULL, NULL, 0, 0, 0, 0, 0, 0, 0, NULL }
+};
+
+static struct ServiceMessage cert_msgtab = {
+  cert_sub, "CERT", 0, 1, 1, 0, IDENTIFIED_FLAG, NS_HELP_CERT_SHORT, 
+  NS_HELP_CERT_LONG, NULL
+};
+
 static struct ServiceMessage access_sub[] = {
   { NULL, "ADD", 0, 1, 1, 0, IDENTIFIED_FLAG, NS_HELP_ACCESS_ADD_SHORT, 
     NS_HELP_ACCESS_ADD_LONG, m_access_add },
@@ -198,13 +221,18 @@ static struct ServiceMessage list_msgtab = {
   NS_HELP_LIST_LONG, m_list
 };
 
-INIT_MODULE(nickserv, "$Revision: 1043 $")
+static struct ServiceMessage status_msgtab = {
+  NULL, "STATUS", 0, 1, 1, 0, USER_FLAG, NS_HELP_STATUS_SHORT,
+  NS_HELP_STATUS_LONG, m_status
+};
+
+INIT_MODULE(nickserv, "$Revision: 1184 $")
 {
   nickserv = make_service("NickServ");
   clear_serv_tree_parse(&nickserv->msg_tree);
   dlinkAdd(nickserv, &nickserv->node, &services_list);
   hash_add_service(nickserv);
-  introduce_client(nickserv->name);
+  nickserv_client = introduce_client(nickserv->name, nickserv->name, TRUE);
   load_language(nickserv->languages, "nickserv.en");
 
   mod_add_servcmd(&nickserv->msg_tree, &drop_msgtab);
@@ -213,6 +241,7 @@ INIT_MODULE(nickserv, "$Revision: 1043 $")
   mod_add_servcmd(&nickserv->msg_tree, &register_msgtab);
   mod_add_servcmd(&nickserv->msg_tree, &set_msgtab);
   mod_add_servcmd(&nickserv->msg_tree, &access_msgtab);
+  mod_add_servcmd(&nickserv->msg_tree, &cert_msgtab);
   mod_add_servcmd(&nickserv->msg_tree, &ghost_msgtab);
   mod_add_servcmd(&nickserv->msg_tree, &link_msgtab);
   mod_add_servcmd(&nickserv->msg_tree, &unlink_msgtab);
@@ -225,11 +254,13 @@ INIT_MODULE(nickserv, "$Revision: 1043 $")
   mod_add_servcmd(&nickserv->msg_tree, &cloakstring_msgtab);
   mod_add_servcmd(&nickserv->msg_tree, &sendpass_msgtab);
   mod_add_servcmd(&nickserv->msg_tree, &list_msgtab);
+  mod_add_servcmd(&nickserv->msg_tree, &status_msgtab);
   
   ns_umode_hook       = install_hook(on_umode_change_cb, ns_on_umode_change);
   ns_nick_hook        = install_hook(on_nick_change_cb, ns_on_nick_change);
   ns_newuser_hook     = install_hook(on_newuser_cb, ns_on_newuser);
   ns_quit_hook        = install_hook(on_quit_cb, ns_on_quit);
+  ns_certfp_hook      = install_hook(on_certfp_cb, ns_on_certfp);
   
   guest_number = 0;
 
@@ -243,10 +274,11 @@ CLEANUP_MODULE
   uninstall_hook(on_nick_change_cb, ns_on_nick_change);
   uninstall_hook(on_newuser_cb, ns_on_newuser);
   uninstall_hook(on_quit_cb, ns_on_quit);
+  uninstall_hook(on_certfp_cb, ns_on_certfp);
   eventDelete(process_enforce_list, NULL);
   eventDelete(process_release_list, NULL);
   serv_clear_messages(nickserv);
-  exit_client(find_client(nickserv->name), &me, "Service unloaded");
+  exit_client(nickserv_client, &me, "Service unloaded");
   unload_languages(nickserv->languages);
   hash_del_service(nickserv);
   ilog(L_DEBUG, "Unloaded nickserv");
@@ -322,7 +354,7 @@ handle_nick_change(struct Service *service, struct Client *client,
     }
     else if(target != client)
     {
-      target->release_to = client;
+      target->release_to = client->name;
       strlcpy(target->release_name, name, NICKLEN);
       guest_user(target);
       reply_user(service, service, client, message, name);
@@ -330,7 +362,6 @@ handle_nick_change(struct Service *service, struct Client *client,
     else
     {
       identify_user(client);
-      db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
       reply_user(service, service, client, message, name);
     }
   }
@@ -340,6 +371,7 @@ handle_nick_change(struct Service *service, struct Client *client,
     send_nick_change(service, client, name);
   }
 
+  dlinkFindDelete(&nick_enforce_list, client);
   SetIdentified(client);
 }
 
@@ -397,7 +429,6 @@ m_register(struct Service *service, struct Client *client,
   {
     client->nickname = nick;
     identify_user(client);
-    db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
 
     reply_user(service, service, client, NS_REG_COMPLETE, client->name);
     global_notice(NULL, "%s!%s@%s registered nick %s\n", client->name, 
@@ -415,6 +446,7 @@ m_drop(struct Service *service, struct Client *client,
   struct Client *target;
   struct Nick *nick = db_find_nick(client->name);
   char *target_nick = NULL;
+  char *channel;
 
   assert(nick != NULL);
 
@@ -490,8 +522,21 @@ m_drop(struct Service *service, struct Client *client,
    * drop
    */
 
-  if(db_delete_nick(client->nickname->id, client->nickname->nickid,
-        client->nickname->pri_nickid)) 
+  /*
+   * If a channel will end up a masterless with this drop, do not allow it.
+   * This sort of violates the nickserv shouldnt deal with channels policy,
+   * but it's a little naive not to realise the bond.
+   */
+  if((channel = check_masterless_channels(client->nickname->id)) != NULL)
+  {
+    reply_user(service, service, client, NS_DROP_FAIL_MASTERLESS, client->name,
+        channel);
+    MyFree(channel);
+    return;
+  }
+
+  if(db_delete_nick(client->nickname->id, client->nickname->pri_nickid,
+        client->nickname->nickid)) 
   {
     if(target != NULL)
     {
@@ -572,7 +617,6 @@ m_identify(struct Service *service, struct Client *client,
   else
   {
     identify_user(client);
-    db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
     reply_user(service, service, client, NS_IDENTIFIED, name);
   }
 }
@@ -1001,6 +1045,86 @@ m_access_del(struct Service *service, struct Client *client, int parc,
   reply_user(service, service, client, NS_ACCESS_DEL, ret);
 }
 
+static void
+m_cert_add(struct Service *service, struct Client *client, int parc, 
+    char *parv[])
+{
+  struct Nick *nick = client->nickname;
+  struct AccessEntry access;
+
+  access.id = nick->id;
+  if(parv[1] == NULL)
+  {
+    if(*client->certfp == '\0')
+    {
+      reply_user(service, service, client, NS_CERT_YOUHAVENONE);
+      return;
+    }
+    access.value = client->certfp;
+  }
+  else
+  {
+    if(strlen(parv[1]) != 40)
+    {
+      reply_user(service, service, client, NS_CERT_INVALID, parv[1]);
+      return;
+    }
+    access.value = parv[1];
+  }
+
+  if(db_list_add(CERT_LIST, (void *)&access))
+    reply_user(service, service, client, NS_CERT_ADD, access.value);
+  else
+    reply_user(service, service, client, NS_CERT_ADDFAIL, access.value);
+}
+
+static void
+m_cert_list(struct Service *service, struct Client *client, int parc, 
+    char *parv[])
+{
+  struct Nick *nick;
+  struct AccessEntry *entry = NULL;
+  void *first, *listptr;
+  int i = 1;
+
+  nick = client->nickname;
+
+  reply_user(service, service, client, NS_CERT_START);
+ 
+  if((listptr = db_list_first(CERT_LIST, nick->id, (void**)&entry)) == NULL)
+  {
+    MyFree(entry);
+    return;
+  }
+
+  first = listptr;
+
+  while(listptr != NULL)
+  {
+    reply_user(service, service, client, NS_CERT_ENTRY, i++, entry->value);
+    MyFree(entry);
+    listptr = db_list_next(listptr, CERT_LIST, (void**)&entry);
+  }
+
+  db_list_done(first);
+}
+
+static void
+m_cert_del(struct Service *service, struct Client *client, int parc, 
+    char *parv[])
+{
+  struct Nick *nick = client->nickname;
+  int index, ret;
+
+  index = atoi(parv[1]);
+  if(index > 0)
+    ret = db_list_del_index(DELETE_NICKCERT_IDX, nick->id, index);
+  else
+    ret = db_list_del(DELETE_NICKCERT, nick->id, parv[1]);
+  
+  reply_user(service, service, client, NS_CERT_DEL, ret);
+}
+
 static void
 m_regain(struct Service *service, struct Client *client, int parc, char *parv[])
 {
@@ -1078,7 +1202,7 @@ m_link(struct Service *service, struct Client *client, int parc, char *parv[])
   free_nick(master_nick);
   free_nick(nick);
 
-  client->nickname = db_find_nick(client->name);
+  client->nickname = db_find_nick(parv[1]);
   assert(client->nickname != NULL);
 }
 
@@ -1572,7 +1696,7 @@ ns_on_nick_change(va_list args)
 
   if(IsEnforce(user))
   {
-    introduce_client(oldnick);
+    introduce_client(oldnick, "Enforced Nickname (/msg nickserv help regain)", FALSE);
     enforcer = find_client(oldnick);
     enforcer->release_time = CurrentTime + (1*60*60);
     dlinkAdd(enforcer, make_dlink_node(), &nick_release_list);
@@ -1582,18 +1706,28 @@ ns_on_nick_change(va_list args)
   /* This user was using a nick wanted elsewhere, release it now */
   if(user->release_to != NULL)
   {
-    struct Client *client = user->release_to;
+    struct Client *client = find_client(user->release_to);
     struct Client *target;
 
-    if((target = find_client(client->nickname->nick)) != NULL)
+    /* in the "real world" it is possible that a user gets deidenfitied in the
+     * mean time, check they are still eligible for this nick.
+     */
+    if(client != NULL && client->nickname != NULL)
     {
-      if(target != client)
-        kill_user(nickserv, target, "This nickname is registered and protected");
+      if((target = find_client(client->nickname->nick)) != NULL)
+      {
+        if(target != client)
+          kill_user(nickserv, target, "This nickname is registered and protected");
+      }
+      send_nick_change(nickserv, client, user->release_name);
+      user->release_to = NULL;
+      memset(user->release_name, 0, sizeof(user->release_name));
+      db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
+      identify_user(client);
     }
     send_nick_change(nickserv, client, user->release_name);
     user->release_to = NULL;
     memset(user->release_name, 0, sizeof(user->release_name));
-    db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
     identify_user(client);
   }
 
@@ -1608,6 +1742,11 @@ ns_on_nick_change(va_list args)
     /* XXX Use unidentify event */
     send_umode(nickserv, user, "-R");
 
+    if(IsSentCert(user))
+    {
+      ClearSentCert(user);
+    }
+
     if(user->nickname)
     {
       oldid = user->nickname->id;
@@ -1642,7 +1781,6 @@ ns_on_nick_change(va_list args)
     if(user->nickname != NULL)
       free_nick(user->nickname);
     user->nickname = nick_p;
-    db_set_string(SET_NICK_LAST_REALNAME, user->nickname->id, user->info);
     identify_user(user);
     return pass_callback(ns_nick_hook, user, oldnick);
   }
@@ -1659,7 +1797,6 @@ ns_on_nick_change(va_list args)
         free_nick(user->nickname);
       user->nickname = nick_p;
       identify_user(user);
-      db_set_string(SET_NICK_LAST_REALNAME, user->nickname->id, user->info);
       reply_user(nickserv, nickserv, user, NS_IDENTIFY_ACCESS, user->name);
     }
   }
@@ -1717,7 +1854,6 @@ ns_on_newuser(va_list args)
       free_nick(newuser->nickname);
     newuser->nickname = nick_p;
     identify_user(newuser);
-    db_set_string(SET_NICK_LAST_REALNAME, newuser->nickname->id, newuser->info);
     return pass_callback(ns_newuser_hook, newuser);
   }
 
@@ -1732,7 +1868,6 @@ ns_on_newuser(va_list args)
         free_nick(newuser->nickname);
       newuser->nickname = nick_p;
       identify_user(newuser);
-      db_set_string(SET_NICK_LAST_REALNAME, newuser->nickname->id, newuser->info);
       reply_user(nickserv, nickserv, newuser, NS_IDENTIFY_ACCESS, newuser->name);
     }
   }
@@ -1782,3 +1917,68 @@ ns_on_quit(va_list args)
 
   return pass_callback(ns_quit_hook, user, comment);
 }
+
+static void *
+ns_on_certfp(va_list args)
+{
+  struct Client *user = va_arg(args, struct Client *);
+  struct Nick *nick = db_find_nick(user->name);
+
+  if(nick == NULL)
+    return pass_callback(ns_certfp_hook, user);
+
+  if(user->nickname != NULL && user->nickname->nickid == nick->nickid) 
+  {
+    /* Already identified for this nick */
+    free_nick(nick);
+    return pass_callback(ns_certfp_hook, user);
+  }
+
+  if(check_list_entry(CERT_LIST, nick->id, user->certfp))
+  {
+    if(user->nickname != NULL)
+      free_nick(user->nickname);
+    user->nickname = nick;
+    identify_user(user);
+    SetSentCert(user);
+    reply_user(nickserv, nickserv, user, NS_IDENTIFY_CERT, user->name);
+  }
+  
+  return pass_callback(ns_certfp_hook, user);
+}
+
+static void
+m_status(struct Service *service, struct Client *client, int parc, char *parv[])
+{
+  struct Client *target;
+  char *name = parv[1];         //already checked # params
+
+  if((target = find_client(name)) != NULL)
+  {
+    if(IsSentCert(target))
+    {
+      reply_user(service, service, client, NS_STATUS_SSL, name);
+      return;  
+    }
+    else if(IsOnAccess(target))
+    {
+      reply_user(service, service, client, NS_STATUS_ACCESS, name);
+      return;  
+    }
+    else if(IsIdentified(target))
+    {
+      reply_user(service, service, client, NS_STATUS_PASS, name);
+      return;  
+    }
+    else
+    {
+      reply_user(service, service, client, NS_STATUS_NOTREG, name);
+      return;  
+    }
+  }
+  else
+  {
+    reply_user(service, service, client, NS_STATUS_OFFLINE, name);
+    return;  
+  }
+}
