@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: oftc.c 991 2007-05-11 10:22:58Z stu $
+ *  $Id: oftc.c 1139 2007-08-20 15:19:25Z swalsh $
  */
 
 #include "stdinc.h"
@@ -31,6 +31,7 @@ static void *oftc_sendmsg_svscloak(va_list);
 static void *oftc_sendmsg_svsnick(va_list);
 //static void *oftc_sendmsg_svsjoin(va_list);
 static void *oftc_identify(va_list);
+static void *oftc_chops_notice(va_list);
 
 static dlink_node *oftc_gnotice_hook;
 static dlink_node *oftc_umode_hook;
@@ -39,6 +40,7 @@ static dlink_node *oftc_svsnick_hook;
 //static dlink_node *oftc_svsjoin_hook;
 static dlink_node *oftc_identify_hook;
 static dlink_node *oftc_connected_hook;
+static dlink_node *oftc_chops_notice_hook;
 
 static void m_pass(struct Client *, struct Client *, int, char *[]);
 static void m_server(struct Client *, struct Client *, int, char *[]);
@@ -50,6 +52,7 @@ static void m_bmask(struct Client *, struct Client *, int, char *[]);
 static void m_svsmode(struct Client *, struct Client *, int, char *[]);
 static void m_eob(struct Client *, struct Client *, int, char *[]);
 static void m_realhost(struct Client *, struct Client *, int, char *[]);
+static void m_certfp(struct Client *, struct Client *, int, char *[]);
 
 static struct Message gnotice_msgtab = {
   "GNOTICE", 0, 0, 3, 0, 0, 0,
@@ -104,6 +107,11 @@ static struct Message realhost_msgtab = {
   { m_realhost, m_realhost }
 };
 
+static struct Message certfp_msgtab = {
+  "CERTFP", 0, 0, 3, 0, 0, 0,
+  { m_certfp, m_certfp }
+};
+
 struct ModeList ModeList[] = {
   { MODE_NOPRIVMSGS,  'n' },
   { MODE_TOPICLIMIT,  't' },
@@ -118,7 +126,7 @@ struct ModeList ModeList[] = {
   { 0, '\0' }
 };
 
-INIT_MODULE(oftc, "$Revision: 991 $")
+INIT_MODULE(oftc, "$Revision: 1139 $")
 {
   oftc_connected_hook  = install_hook(connected_cb, oftc_server_connected);
   oftc_gnotice_hook   = install_hook(send_gnotice_cb, oftc_sendmsg_gnotice);
@@ -127,6 +135,7 @@ INIT_MODULE(oftc, "$Revision: 991 $")
 //  oftc_svsjoin_hook   = install_hook(send_nick_cb, oftc_sendmsg_svsjoin);
   oftc_svsnick_hook   = install_hook(send_nick_cb, oftc_sendmsg_svsnick);
   oftc_identify_hook  = install_hook(on_identify_cb, oftc_identify);
+  oftc_chops_notice_hook = install_hook(send_chops_notice_cb, oftc_chops_notice);
   mod_add_cmd(&gnotice_msgtab);
   mod_add_cmd(&pass_msgtab);
   mod_add_cmd(&server_msgtab);
@@ -138,6 +147,7 @@ INIT_MODULE(oftc, "$Revision: 991 $")
   mod_add_cmd(&svsmode_msgtab);
   mod_add_cmd(&eob_msgtab);
   mod_add_cmd(&realhost_msgtab);
+  mod_add_cmd(&certfp_msgtab);
 }
 
 CLEANUP_MODULE
@@ -153,6 +163,7 @@ CLEANUP_MODULE
   mod_del_cmd(&svsmode_msgtab);
   mod_del_cmd(&eob_msgtab);
   mod_del_cmd(&realhost_msgtab);
+  mod_del_cmd(&certfp_msgtab);
 
   uninstall_hook(send_gnotice_cb, oftc_sendmsg_gnotice);
   uninstall_hook(send_umode_cb, oftc_sendmsg_svsmode);
@@ -236,6 +247,25 @@ m_realhost(struct Client *client, struct Client *source, int parc, char *parv[])
   strlcpy(nclient->realhost, parv[2], sizeof(nclient->realhost));
 }
 
+static void
+m_certfp(struct Client *client, struct Client *source, int parc, char *parv[])
+{
+  struct Client *target = NULL;
+
+  ilog(L_DEBUG, "CERTFP for %s is %s", parv[1], parv[2]);
+
+  target = find_person(source, parv[1]);
+
+  if(target == NULL)
+  {
+    ilog(L_INFO, "Got CERTFP for %s but we don't know the client", parv[1]);
+    return;
+  }
+
+  strlcpy(target->certfp, parv[2], sizeof(target->certfp));
+  execute_callback(on_certfp_cb, target);
+}
+
 static void
 m_eob(struct Client *client, struct Client *source, int parc, char *parv[])
 {
@@ -515,7 +545,15 @@ oftc_server_connected(va_list args)
   {
     struct Service *service = ptr->data;
 
-    introduce_client(service->name);
+    introduce_client(service->name, service->name, TRUE);
+  }
+
+  ptr = NULL;
+
+  DLINK_FOREACH(ptr, me.server_list.head)
+  {
+    struct Client *server = ptr->data;
+    introduce_server(server->name, server->info);
   }
 
   return NULL;
@@ -588,3 +626,29 @@ oftc_sendmsg_svsnick(va_list args)
   
   return pass_callback(oftc_svsnick_hook, uplink, user, newnick);
 }
+
+static void *
+oftc_chops_notice(va_list args)
+{
+  struct Client *uplink = va_arg(args, struct Client *);
+  struct Service *source = va_arg(args, struct Service *);
+  struct Channel *chptr = va_arg(args, struct Channel *);
+  char          *notice = va_arg(args, char *);
+  dlink_node *ptr;
+  struct Membership *ms;
+  struct Client *target;
+
+  DLINK_FOREACH(ptr, chptr->members.head)
+  {
+    ms = ptr->data;
+    target = ms->client_p;
+
+    if ((ms->flags & CHFL_CHANOP) == 0)
+      continue;
+
+    sendto_server(uplink, ":%s NOTICE %s :%s", source->name, target->name, 
+        notice);
+  }
+
+  return pass_callback(oftc_chops_notice_hook, uplink, source, chptr, notice);
+}
