@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: operserv.c 1086 2007-06-26 14:06:21Z swalsh $
+ *  $Id: operserv.c 1154 2007-08-23 18:34:52Z tjfontaine $
  */
 
 #include "stdinc.h"
@@ -31,8 +31,12 @@
 static struct Service *operserv = NULL;
 
 static dlink_node *os_newuser_hook;
+static dlink_node *os_db_init_hook;
+static dlink_node *os_quit_hook;
 
 static void *os_on_newuser(va_list);
+static void *os_on_db_init(va_list);
+static void *os_on_quit(va_list);
 
 static void m_help(struct Service *, struct Client *, int, char *[]);
 static void m_raw(struct Service *, struct Client *, int, char *[]);
@@ -47,6 +51,9 @@ static void m_admin_del(struct Service *, struct Client *, int, char *[]);
 static void m_akill_add(struct Service *, struct Client *, int, char *[]);
 static void m_akill_list(struct Service *, struct Client *, int, char *[]);
 static void m_akill_del(struct Service *, struct Client *, int, char *[]);
+static void m_jupes_add(struct Service *, struct Client *, int, char *[]);
+static void m_jupes_list(struct Service *, struct Client *, int, char *[]);
+static void m_jupes_del(struct Service *, struct Client *, int, char *[]);
 
 static struct ServiceMessage help_msgtab = {
   NULL, "HELP", 0, 0, 2, 0, OPER_FLAG, OS_HELP_SHORT, OS_HELP_LONG, m_help
@@ -108,17 +115,34 @@ static struct ServiceMessage set_msgtab = {
   m_operserv_notimp
 };
 
-INIT_MODULE(operserv, "$Revision: 1086 $")
+static struct ServiceMessage jupes_subs[] = {
+  { NULL, "ADD", 0, 1, 3, SFLG_NOMAXPARAM, OPER_FLAG, OS_JUPES_ADD_HELP_SHORT,
+    OS_JUPES_ADD_HELP_LONG, m_jupes_add },
+  { NULL, "LIST", 0, 0, 0, 0, OPER_FLAG, OS_JUPES_LIST_HELP_SHORT,
+    OS_JUPES_LIST_HELP_LONG, m_jupes_list },
+  { NULL, "DEL", 0, 1, 1, 0, OPER_FLAG, OS_JUPES_DEL_HELP_SHORT,
+    OS_JUPES_DEL_HELP_LONG, m_jupes_del },
+  { NULL, NULL, 0, 0, 0, 0, 0, 0, 0, NULL }
+};
+
+static struct ServiceMessage jupes_msgtab = {
+  jupes_subs, "JUPES", 0, 2, 2, 0, OPER_FLAG, OS_JUPES_HELP_SHORT,
+  OS_JUPES_HELP_LONG, NULL
+};
+
+INIT_MODULE(operserv, "$Revision: 1154 $")
 {
   operserv = make_service("OperServ");
   clear_serv_tree_parse(&operserv->msg_tree);
   dlinkAdd(operserv, &operserv->node, &services_list);
   hash_add_service(operserv);
-  introduce_client(operserv->name);
+  introduce_client(operserv->name, operserv->name, TRUE);
 
   load_language(operserv->languages, "operserv.en");
 
   os_newuser_hook = install_hook(on_newuser_cb, os_on_newuser);
+  os_db_init_hook = install_hook(on_db_init_cb, os_on_db_init);
+  os_quit_hook = install_hook(on_quit_cb, os_on_quit);
 
   mod_add_servcmd(&operserv->msg_tree, &help_msgtab);
   mod_add_servcmd(&operserv->msg_tree, &mod_msgtab);
@@ -127,11 +151,14 @@ INIT_MODULE(operserv, "$Revision: 1086 $")
   mod_add_servcmd(&operserv->msg_tree, &akill_msgtab);
   mod_add_servcmd(&operserv->msg_tree, &set_msgtab);
   mod_add_servcmd(&operserv->msg_tree, &raw_msgtab);
+  mod_add_servcmd(&operserv->msg_tree, &jupes_msgtab);
 }
 
 CLEANUP_MODULE
 {
   uninstall_hook(on_newuser_cb, os_on_newuser);
+  uninstall_hook(on_db_init_cb, os_on_db_init);
+  uninstall_hook(on_quit_cb, os_on_quit);
 
   serv_clear_messages(operserv);
 
@@ -139,6 +166,29 @@ CLEANUP_MODULE
   ilog(L_DEBUG, "Unloaded operserv");
 }
 
+static void *
+os_on_db_init(va_list param)
+{
+  struct JupeEntry *jupe;
+  void *handle, *first;
+
+  first = handle = db_list_first(JUPES_LIST, 0, (void**)&jupe);
+  while(handle != NULL)
+  {
+    introduce_server(jupe->name, jupe->reason);
+    ilog(L_DEBUG, "JUPE %s [%s]", jupe->name, jupe->reason);
+    free_jupeentry(jupe);
+    jupe = NULL;
+    handle = db_list_next(handle, JUPES_LIST, (void**)&jupe);
+  }
+  if(first)
+    db_list_done(first);
+
+  free_jupeentry(jupe);
+
+  return pass_callback(os_db_init_hook, param);
+}
+
 static void
 m_help(struct Service *service, struct Client *client,
         int parc, char *parv[])
@@ -208,13 +258,6 @@ m_mod_reload(struct Service *service, struct Client *client,
 
   mbn = basename(parm);
   module = find_module(mbn, 0);
-  if(irccmp(module->name, service->name) == 0)
-  {
-    ilog(L_NOTICE, "%s tried to reload %s.  Can't be done because it's me!",
-        client->name, service->name);
-    reply_user(service, service, client, OS_MOD_CANTRELOAD, parm);
-    return;
-  }
   if (module == NULL)
   {
     ilog(L_NOTICE, "Module %s reload requested by %s, but failed because not loaded",
@@ -222,6 +265,13 @@ m_mod_reload(struct Service *service, struct Client *client,
     reply_user(service, service, client, OS_MOD_NOTLOADED, parm, client->name);
     return;
   }
+  if(irccmp(module->name, service->name) == 0)
+  {
+    ilog(L_NOTICE, "%s tried to reload %s.  Can't be done because it's me!",
+        client->name, service->name);
+    reply_user(service, service, client, OS_MOD_CANTRELOAD, parm);
+    return;
+  }
   ilog(L_NOTICE, "Reloading %s by request of %s", parm, client->name);
   reply_user(service, service, client, OS_MOD_RELOADING, parm, client->name);
   unload_module(module);
@@ -497,6 +547,7 @@ m_akill_del(struct Service *service, struct Client *client,
 {
   int ret = 0;
   struct ServiceBan *akill;
+  char *expire_time, *set_time;
 
   if((akill = db_find_akill(parv[1])) == NULL)
   {
@@ -506,6 +557,24 @@ m_akill_del(struct Service *service, struct Client *client,
   
   ret = db_list_del(DELETE_AKILL, 0, parv[1]);
   remove_akill(service, akill);
+  if(ret)
+  {
+    /* XXX Have to copy the string since smalldate uses a static buffer! */
+    DupString(set_time, smalldate(akill->time_set));
+    if(akill->duration > 0)
+      DupString(expire_time, smalldate(akill->time_set + akill->duration));
+    else
+      expire_time = "never";
+
+    ilog(L_NOTICE, "%s removed akill on %s.  Akill was set %s and would have "
+        "expired %s. (%s)", client->name, akill->mask, set_time, expire_time,
+        akill->reason);
+
+    if(akill->duration > 0)
+      MyFree(expire_time);
+    MyFree(set_time);
+  }
+  free_serviceban(akill);
   reply_user(service, service, client, OS_AKILL_DEL, ret);
 }
 
@@ -527,3 +596,128 @@ os_on_newuser(va_list args)
 
   return pass_callback(os_newuser_hook, newuser);
 }
+
+static void
+m_jupes_add(struct Service *service, struct Client *client,
+    int parc, char *parv[])
+{
+  struct JupeEntry *entry = db_find_jupe(parv[1]);
+  char reason[IRC_BUFSIZE+1] = "Jupitered: No Reason";
+  int ret = 0;
+
+  if(entry != NULL)
+  {
+    reply_user(service, service, client, OS_JUPES_ALREADY, entry->name, entry->reason);
+    free_jupeentry(entry);
+    return;
+  }
+
+  if(parc >= 2)
+    join_params(reason, parc-1, &parv[2]);
+
+  entry = MyMalloc(sizeof(struct JupeEntry));
+
+  DupString(entry->name, parv[1]);
+  DupString(entry->reason, reason);
+
+  entry->setter = client->nickname->id;
+
+  ret = db_list_add(JUPES_LIST, entry);
+
+  if(ret)
+  {
+    introduce_server(entry->name, entry->reason);
+    reply_user(service, service, client, OS_JUPES_ADDED, entry->name, entry->reason);
+    ilog(L_NOTICE, "%s Jupitered %s [%s]", client->name, entry->name, entry->reason);
+  }
+  else
+  {
+    reply_user(service, service, client, OS_JUPES_ADD_FAILED, entry->name);
+    ilog(L_NOTICE, "%s Failed to Jupiter %s", client->name, entry->name);
+  }
+
+  free_jupeentry(entry);
+}
+
+static void
+m_jupes_list(struct Service *service, struct Client *client,
+    int parc, char *parv[])
+{
+  struct JupeEntry *jupe;
+  void *handle, *first;
+
+  int i = 1;
+
+  first = handle = db_list_first(JUPES_LIST, 0, (void**)&jupe);
+  while(handle != NULL)
+  {
+    char *setter = db_get_nickname_from_id(jupe->setter);
+
+    reply_user(service, service, client, OS_JUPES_LIST, i++, jupe->name,
+      jupe->reason, setter);
+
+    if(jupe->setter > 0)
+      MyFree(setter);
+
+    if(jupe != NULL)
+    {
+      free_jupeentry(jupe);
+      jupe = NULL;
+    }
+
+    handle = db_list_next(handle, JUPES_LIST, (void**)&jupe);
+  }
+
+  if(first)
+    db_list_done(first);
+
+  reply_user(service, service, client, OS_JUPES_LIST_END);
+
+  free_jupeentry(jupe);
+}
+
+static void
+m_jupes_del(struct Service *service, struct Client *client,
+    int parc, char *parv[])
+{
+  int ret;
+
+  ret = db_list_del(DELETE_JUPES_NAME, 0, parv[1]);
+
+  squit_server(parv[1], "UnJupitered");
+
+  if(ret > 0)
+  {
+    ilog(L_NOTICE, "%s Removed Jupiter for %s", client->name, parv[1]);
+    reply_user(service, service, client, OS_JUPES_DELETED, parv[1]);
+  }
+  else
+  {
+    ilog(L_NOTICE, "%s Failed to remove Jupiter for %s", client->name, parv[1]);
+    reply_user(service, service, client, OS_JUPES_DEL_FAILED, parv[1]);
+  }
+}
+
+static void *
+os_on_quit(va_list param)
+{
+  struct Client *server = va_arg(param, struct Client *);
+  char *reason          = va_arg(param, char *);
+  int ret;
+
+  if(IsServer(server) && IsMe(server->servptr))
+  {
+    ret = db_list_del(DELETE_JUPES_NAME, 0, server->name);
+
+    if(ret > 0)
+    {
+      ilog(L_NOTICE, "Removed Jupiter for %s", server->name);
+    }
+    else
+    {
+      ilog(L_NOTICE, "Failed to remove Jupiter for %s", server->name);
+    }
+  }
+
+  return pass_callback(os_quit_hook, server, reason);
+}
