@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: channel_mode.c 1059 2007-05-29 19:50:39Z tjfontaine $
+ *  $Id: channel_mode.c 1122 2007-07-20 16:17:43Z swalsh $
  */
 
 #include "stdinc.h"
@@ -239,7 +239,7 @@ del_id(struct Channel *chptr, char *banid, int type)
 }
 
 /* XXX This needs to come from the protocol module!! */
-static const struct mode_letter
+/*static const struct mode_letter
 {
   const unsigned int mode;
   const unsigned char letter;
@@ -252,7 +252,8 @@ static const struct mode_letter
   { MODE_TOPICLIMIT, 't' },
   { MODE_NOCOLOR,    'c' },
   { 0, '\0' }
-};
+};*/
+static const struct ModeList *flags;
 
 /* channel_modes()
  *
@@ -523,14 +524,6 @@ chm_invex(struct Client *client_p, struct Client *source_p,
 {
   char *mask = NULL;
 
-  if (alev < CHACCESS_HALFOP)
-  {
-    if (!(*errors & SM_ERR_NOOPS))
-      ilog(L_DEBUG, "Invex from non chop");
-    *errors |= SM_ERR_NOOPS;
-    return;
-  }
-
   mask = nuh_mask[*parn];
   memcpy(mask, parv[*parn], sizeof(nuh_mask[*parn]));
   ++*parn;
@@ -834,12 +827,6 @@ chm_limit(struct Client *client_p, struct Client *source_p,
   int i, limit;
   char *lstr;
 
-  if (alev < CHACCESS_HALFOP)
-  {
-    *errors |= SM_ERR_NOOPS;
-    return;
-  }
-
   if (dir == MODE_QUERY)
     return;
 
@@ -895,12 +882,6 @@ chm_key(struct Client *client_p, struct Client *source_p,
   int i;
   char *key;
 
-  if (alev < CHACCESS_HALFOP)
-  {
-    *errors |= SM_ERR_NOOPS;
-    return;
-  }
-
   if (dir == MODE_QUERY)
     return;
 
@@ -975,13 +956,13 @@ static struct ChannelMode ModeTable[255] =
   {chm_nosuch, NULL},                             /* J */
   {chm_nosuch, NULL},                             /* K */
   {chm_nosuch, NULL},                             /* L */
-  {chm_nosuch, NULL},                             /* M */
+  {chm_simple, (void*)MODE_REGONLYTALK},          /* M */
   {chm_nosuch, NULL},                             /* N */
   {chm_nosuch, NULL},                             /* O */
   {chm_nosuch, NULL},                             /* P */
   {chm_nosuch, NULL},                             /* Q */
-  {chm_nosuch, NULL},                             /* R */
-  {chm_nosuch, NULL},                             /* S */
+  {chm_simple, (void*)MODE_REGONLY},              /* R */
+  {chm_simple, (void*)MODE_SSLONLY},              /* S */
   {chm_nosuch, NULL},                             /* T */
   {chm_nosuch, NULL},                             /* U */
   {chm_nosuch, NULL},                             /* V */
@@ -1087,6 +1068,7 @@ set_channel_mode(struct Client *client_p, struct Client *source_p, struct Channe
   int alevel = 0, errors = 0;
   char *ml = parv[0], c;
   int table_position;
+  char *oldparam;
 
   mode_count = 0;
   mode_limit = 0;
@@ -1112,13 +1094,14 @@ set_channel_mode(struct Client *client_p, struct Client *source_p, struct Channe
           table_position = 0;
         else
           table_position = c - 'A' + 1;
-       ModeTable[table_position].func(client_p, source_p, chptr,
+        oldparam = parv[parn];
+        ModeTable[table_position].func(client_p, source_p, chptr,
                                        parc, &parn,
                                        parv, &errors, alevel, dir, c,
                                        ModeTable[table_position].d,
                                        chname);
        if(dir == MODE_ADD || dir == MODE_DEL)
-         execute_callback(on_cmode_change_cb, source_p, chptr, dir, c, parv[parn]);
+         execute_callback(on_cmode_change_cb, source_p, chptr, dir, c, oldparam);
        break;
     }
   }
