@@ -18,7 +18,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: database.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: database.c 1034 2007-05-17 10:28:01Z swalsh $
  */
 
 #include "stdinc.h"
@@ -62,9 +62,6 @@ verify_database(va_list args)
   if(!Database.username[0])
     parse_fatal("username= field missing in database{} section");
 
-  if(!Database.password[0])
-    parse_fatal("password= field missing in database{} section");
-
   return pass_callback(hverify);
 }
 
