@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: modules.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: modules.c 1055 2007-05-24 22:21:32Z swalsh $
  */
 #include "stdinc.h"
 #include "conf/conf.h"
@@ -291,7 +291,7 @@ unload_module(struct Module *mod)
   }
 #endif
 
-  if (mod != NULL && mod->handle != NULL)
+  if (mod != NULL && mod->handle != NULL && !ServicesState.keepmodules)
     modunload(mod->handle);
 #endif
 }
