@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: dbm.c 1083 2007-06-25 21:55:10Z tjfontaine $
+ *  $Id: dbm.c 1185 2007-10-30 23:14:22Z swalsh $
  */
 
 #include "stdinc.h"
@@ -28,7 +28,6 @@
 
 #define LOG_BUFSIZE 2048
 
-struct Callback *on_nick_drop_cb;
 static FBFILE *db_log_fb;
 
 static void expire_sentmail(void *);
@@ -56,10 +55,11 @@ query_t queries[QUERY_COUNT] = {
   { DELETE_ACCOUNT, "DELETE FROM account WHERE id=?d", NULL, EXECUTE },
   { INSERT_NICKACCESS, "INSERT INTO account_access (account_id, entry) VALUES(?d, ?v)", 
     NULL, EXECUTE },
-  { GET_NICKACCESS, "SELECT id, entry FROM account_access WHERE account_id=?d", NULL, QUERY },
+  { GET_NICKACCESS, "SELECT id, entry FROM account_access WHERE account_id=?d ORDER BY id", NULL, QUERY },
   { GET_ADMINS, "SELECT nick FROM account,nickname WHERE flag_admin=true AND "
-    "account.primary_nick = nickname.id ORDER BY nick", NULL, QUERY },
-  { GET_AKILLS, "SELECT akill.id, setter, mask, reason, time, duration FROM akill", 
+    "account.primary_nick = nickname.id ORDER BY lower(nick)", NULL, QUERY },
+  /* XXX: ORDER BY missing here */
+  { GET_AKILLS, "SELECT akill.id, setter, mask, reason, time, duration FROM akill ORDER BY akill.id",
     NULL, QUERY },
   { GET_CHAN_ACCESSES, "SELECT channel_access.id, channel_access.channel_id, "
       "channel_access.account_id, channel_access.level FROM "
@@ -93,7 +93,7 @@ query_t queries[QUERY_COUNT] = {
   { SET_NICK_PASSWORD, "UPDATE account SET password=?v WHERE id=?d", NULL, EXECUTE },
   { SET_NICK_URL, "UPDATE account SET url=?v WHERE id=?d", NULL, EXECUTE },
   { SET_NICK_EMAIL, "UPDATE account SET email=?v WHERE id=?d", NULL, EXECUTE },
-  { SET_NICK_CLOAK, "UPDATE account SET cloak=?v WHERE id=?d", NULL, EXECUTE },
+  { SET_NICK_CLOAK, "UPDATE account SET cloak=lower(?v) WHERE id=?d", NULL, EXECUTE },
   { SET_NICK_LAST_QUIT, "UPDATE account SET last_quit_msg=?v WHERE id=?d", NULL, EXECUTE },
   { SET_NICK_LAST_HOST, "UPDATE account SET last_host=?v WHERE id=?d", NULL, EXECUTE },
   { SET_NICK_LAST_REALNAME, "UPDATE account SET last_realname=?v where id=?d", NULL, EXECUTE },
@@ -109,9 +109,9 @@ query_t queries[QUERY_COUNT] = {
     EXECUTE },
   { DELETE_ALL_NICKACCESS, "DELETE FROM account_access WHERE account_id=?d", NULL, EXECUTE },
   { DELETE_NICKACCESS_IDX, "DELETE FROM account_access WHERE id = "
-          "(SELECT id FROM account_access AS a WHERE ?d = "
-          "(SELECT COUNT(id)+1 FROM account_access AS b WHERE b.id < a.id AND "
-          "b.account_id = ?d) AND account_id = ?d)", NULL, EXECUTE },
+          "(SELECT a.id FROM account_access AS a WHERE ?d = "
+          "(SELECT COUNT(b.id)+1 FROM account_access AS b WHERE b.id < a.id AND "
+          "b.account_id = ?d) AND a.account_id = ?d)", NULL, EXECUTE },
   { SET_NICK_LINK, "UPDATE nickname SET account_id=?d WHERE account_id=?d", NULL, EXECUTE },
   { SET_NICK_LINK_EXCLUDE, "UPDATE nickname SET account_id=?d WHERE account_id=?d AND id=?d", NULL, EXECUTE },
   { INSERT_NICK_CLONE, "INSERT INTO account (primary_nick, password, salt, url, email, cloak, " 
@@ -154,9 +154,8 @@ query_t queries[QUERY_COUNT] = {
     "time, duration) VALUES (?d, ?d, ?d, ?v, ?d, ?d)", NULL, EXECUTE },
   { INSERT_AKICK_MASK, "INSERT INTO channel_akick (channel_id, setter, reason, mask, "
     "time, duration) VALUES (?d, ?d, ?v, ?v, ?d, ?d)", NULL, EXECUTE },
-  { GET_AKICKS, "SELECT channel_akick.id, channel.channel, target, setter, mask, reason, time, duration FROM "
-    "channel_akick, channel WHERE channel_id=?d AND channel.id=channel_id", 
-    NULL, QUERY },
+  { GET_AKICKS, "SELECT channel_akick.id, channel_id, target, setter, mask, reason, time, duration FROM "
+    "channel_akick WHERE channel_id=?d ORDER BY channel_akick.id", NULL, QUERY },
   { DELETE_AKICK_IDX, "DELETE FROM channel_akick WHERE id = "
           "(SELECT id FROM channel_akick AS a WHERE ?d = "
           "(SELECT COUNT(id)+1 FROM channel_akick AS b WHERE b.id < a.id AND "
@@ -167,16 +166,16 @@ query_t queries[QUERY_COUNT] = {
     "FROM nickname WHERE lower(nick)=lower(?v))", NULL, EXECUTE },
   { SET_NICK_MASTER, "UPDATE account SET primary_nick=?d WHERE id=?d", NULL, EXECUTE },
   { DELETE_AKILL, "DELETE FROM akill WHERE mask=?v", NULL, EXECUTE },
-  { GET_CHAN_MASTER_COUNT, "SELECT COUNT(id) FROM channel_access WHERE channel_id=?d AND level=?d",
+  { GET_CHAN_MASTER_COUNT, "SELECT COUNT(id) FROM channel_access WHERE channel_id=?d AND level=4",
     NULL, QUERY },
-  { GET_NICK_LINKS, "SELECT nick FROM nickname WHERE account_id=?d", NULL, QUERY },
-  { GET_NICK_CHAN_INFO, "SELECT channel, level FROM "
+  { GET_NICK_LINKS, "SELECT nick FROM nickname WHERE account_id=?d ORDER BY lower(nick)", NULL, QUERY },
+  { GET_NICK_CHAN_INFO, "SELECT channel.id, channel, level FROM "
     "channel, channel_access WHERE "
       "channel.id=channel_access.channel_id AND channel_access.account_id=?d "
       "ORDER BY lower(channel.channel)", NULL, QUERY },
   { GET_CHAN_MASTERS, "SELECT nick FROM account, nickname, channel_access WHERE channel_id=?d "
     "AND level=4 AND channel_access.account_id=account.id AND "
-      "account.primary_nick=nickname.id", NULL, QUERY },
+      "account.primary_nick=nickname.id ORDER BY lower(nick)", NULL, QUERY },
   { DELETE_ACCOUNT_CHACCESS, "DELETE FROM channel_access WHERE account_id=?d", NULL, EXECUTE },
   { DELETE_DUPLICATE_CHACCESS, "DELETE FROM channel_access WHERE "
       "(account_id=?d AND level <= (SELECT level FROM channel_access AS x WHERE"
@@ -197,17 +196,40 @@ query_t queries[QUERY_COUNT] = {
     QUERY },
   { DELETE_EXPIRED_SENT_MAIL, "DELETE FROM sent_mail WHERE sent + ?d < ?d", NULL, EXECUTE },
   { GET_NICKS, "SELECT nick FROM account, nickname WHERE account.id=nickname.account_id AND "
-       "account.flag_private='f'", NULL, QUERY },
-  { GET_NICKS_OPER, "SELECT nick FROM nickname", NULL, QUERY },
-  { GET_FORBIDS, "SELECT nick FROM forbidden_nickname", NULL, QUERY },
-  { GET_CHANNELS, "SELECT channel FROM channel WHERE flag_private='f'", NULL, QUERY },
-  { GET_CHANNELS_OPER, "SELECT channel FROM channel", NULL, QUERY },
-  { GET_CHANNEL_FORBID_LIST, "SELECT channel FROM forbidden_channel", NULL, QUERY },
+       "account.flag_private='f' ORDER BY lower(nick)", NULL, QUERY },
+  { GET_NICKS_OPER, "SELECT nick FROM nickname ORDER BY lower(nick)", NULL, QUERY },
+  { GET_FORBIDS, "SELECT nick FROM forbidden_nickname ORDER BY lower(nick)", NULL, QUERY },
+  { GET_CHANNELS, "SELECT channel FROM channel WHERE flag_private='f' ORDER BY lower(channel)", NULL, QUERY },
+  { GET_CHANNELS_OPER, "SELECT channel FROM channel ORDER BY lower(channel)", NULL, QUERY },
+  { GET_CHANNEL_FORBID_LIST, "SELECT channel FROM forbidden_channel ORDER BY lower(channel)", NULL, QUERY },
   { SAVE_NICK, "UPDATE account SET url=?v, email=?v, cloak=?v, flag_enforce=?B, "
     "flag_secure=?B, flag_verified=?B, flag_cloak_enabled=?B, "
       "flag_admin=?B, flag_email_verified=?B, flag_private=?B, language=?d, "
       "last_host=?v, last_realname=?v, last_quit_msg=?v, last_quit_time=?d "
       "WHERE id=?d", NULL, EXECUTE },
+  { INSERT_NICKCERT, "INSERT INTO account_fingerprint (account_id, fingerprint) "
+    "VALUES(?d, upper(?v))", NULL, EXECUTE },
+  { GET_NICKCERT, "SELECT id, fingerprint FROM account_fingerprint WHERE "
+    "account_id=?d ORDER BY id", NULL, QUERY },
+  { DELETE_NICKCERT, "DELETE FROM account_fingerprint WHERE "
+    "account_id=?d AND fingerprint=upper(?v)", NULL, EXECUTE },
+  { DELETE_NICKCERT_IDX, "DELETE FROM account_fingerprint WHERE id = "
+          "(SELECT id FROM account_fingerprint AS a WHERE ?d = "
+          "(SELECT COUNT(id)+1 FROM account_fingerprint AS b WHERE b.id < a.id AND "
+          "b.account_id = ?d) AND account_id = ?d)", NULL, EXECUTE },
+  { DELETE_ALL_NICKACCESS, "DELETE FROM account_fingerprint WHERE "
+    "account_id=?d", NULL, EXECUTE },
+  { INSERT_JUPE, "INSERT INTO jupes (setter, name, reason) VALUES(?d, ?v, ?v)",
+    NULL, EXECUTE },
+  { GET_JUPES, "SELECT id, name, reason, setter FROM jupes ORDER BY id", NULL, QUERY },
+  { DELETE_JUPES_NAME, "DELETE FROM jupes WHERE lower(name) = lower(?v)", NULL,
+    EXECUTE },
+  { FIND_JUPE, "SELECT id, name, reason, setter FROM jupes WHERE "
+    "lower(name) = lower(?v)", NULL, QUERY },
+  { COUNT_CHANNEL_ACCESS_LIST, "SELECT COUNT(*) FROM channel_access "
+    "JOIN account ON channel_access.account_id=account.id "
+    "JOIN nickname ON account.primary_nick=nickname.id WHERE channel_id=?d",
+    NULL, QUERY },
 };
 
 void
@@ -226,13 +248,15 @@ init_db()
     len += strlen(Database.hostname);
   len += strlen(port);
   len += strlen(":::");
+  len += strlen(Database.dbname);
   len++;
 
   dbstr = MyMalloc(len);
-  snprintf(dbstr, len-1, "%s:%s:%s:%s", Database.driver, 
+  snprintf(dbstr, len, "%s:%s:%s:%s", Database.driver, 
       Database.hostname == NULL ? "" : Database.hostname, port, Database.dbname);
 
   Database.yada = yada_init(dbstr, 0);
+
   MyFree(dbstr);
 
   snprintf(logpath, LOG_BUFSIZE, "%s/%s", LOGDIR, Logging.sqllog);
@@ -264,6 +288,28 @@ cleanup_db()
   fbclose(db_log_fb);
 }
 
+void
+db_reopen_log()
+{
+  char logpath[LOG_BUFSIZE+1];
+
+  if(db_log_fb != NULL)
+  {
+    fbclose(db_log_fb);
+    db_log_fb = NULL;
+  }
+
+  snprintf(logpath, LOG_BUFSIZE, "%s/%s", LOGDIR, Logging.sqllog);
+  if(db_log_fb == NULL)
+  {
+    if(Logging.sqllog[0] != '\0' && (db_log_fb = fbopen(logpath, "r")) != NULL)
+    {
+      fbclose(db_log_fb);
+      db_log_fb = fbopen(logpath, "a");
+    }
+  }
+}
+
 void
 db_log(const char *format, ...)
 {
@@ -292,14 +338,14 @@ db_load_driver()
 
   if(Database.yada->connect(Database.yada, Database.username, 
         Database.password) == 0)
-    db_log("db: Failed to connect to database %s\n", Database.yada->errmsg);
+    db_log("db: Failed to connect to database %s", Database.yada->errmsg);
   else
-    db_log("db: Database connection succeeded.\n");
+    db_log("db: Database connection succeeded.");
 
   for(i = 0; i < QUERY_COUNT; i++)
   {
     query_t *query = &queries[i];
-    db_log("Prepare %d: %s\n", i, query->name);
+    db_log("Prepare %d: %s", i, query->name);
     if(query->name == NULL)
       continue;
     query->rc = Prepare((char*)query->name, 0);
@@ -309,6 +355,8 @@ db_load_driver()
 
   eventAdd("Expire sent mail", expire_sentmail, NULL, 60); 
   eventAdd("Expire akills", expire_akills, NULL, 60); 
+
+  execute_callback(on_db_init_cb);
 }
 
 void
@@ -331,7 +379,7 @@ db_try_reconnect()
       for(i = 0; i < QUERY_COUNT; i++)
       {
         query_t *query = &queries[i];
-        db_log("%d: %s\n", i, query->name);
+        db_log("%d: %s", i, query->name);
         if(query->name == NULL)
           continue;
         query->rc = Prepare((char*)query->name, 0);
@@ -356,7 +404,7 @@ db_try_reconnect()
   query_t *__query;                                                   \
                                                                       \
   __query = &queries[__id];                                           \
-  db_log("db_query: %d %s\n", __id, __query->name);                   \
+  db_log("db_query: %d %s", __id, __query->name);                     \
   assert(__query->type == QUERY);                                     \
   assert(__query->rc);                                                \
   assert(__query->index == query_id);                                 \
@@ -369,7 +417,7 @@ db_try_reconnect()
       db_try_reconnect();                                             \
       db_log("Query failed because server went away, reconnected.");  \
     }                                                                 \
-    db_log(L_CRIT, "db_query: %d Failed: %s", __id, Database.yada->errmsg); \
+    db_log("db_query: %d Failed: %s", __id, Database.yada->errmsg);   \
   }                                                                   \
                                                                       \
   ret = __result;                                                     \
@@ -382,7 +430,7 @@ db_try_reconnect()
   query_t *__query;                                                   \
                                                                       \
   __query = &queries[__id];                                           \
-  db_log("db_exec: %d %s\n", __id, __query->name);                    \
+  db_log("db_exec: %d %s", __id, __query->name);                      \
   assert(__query->type == EXECUTE);                                   \
   assert(__query->rc);                                                \
                                                                       \
@@ -394,7 +442,7 @@ db_try_reconnect()
       db_try_reconnect();                                             \
       db_log("Exec failed because server went away, reconnected.");   \
     }                                                                 \
-    db_log("db_exec: %d Failed: %s\n", __id, Database.yada->errmsg);  \
+    db_log("db_exec: %d Failed: %s", __id, Database.yada->errmsg);    \
   }                                                                   \
                                                                       \
   ret = __result;                                                     \
@@ -427,7 +475,7 @@ db_find_nick(const char *nick)
 
   if(Fetch(rc, brc) == 0)
   {
-    db_log("db_find_nick: '%s' not found.\n", nick);
+    db_log("db_find_nick: '%s' not found.", nick);
     Free(brc);
     Free(rc);
     MyFree(nick_p);
@@ -447,7 +495,7 @@ db_find_nick(const char *nick)
   DupString(nick_p->last_realname, nick_p->last_realname);
   DupString(nick_p->last_quit, nick_p->last_quit);
 
-  db_log("db_find_nick: Found nick %s(asked for %s)\n", nick_p->nick, nick);
+  db_log("db_find_nick: Found nick %s(asked for %s)", nick_p->nick, nick);
 
   Free(brc);
   Free(rc);
@@ -469,7 +517,7 @@ db_get_nickname_from_id(unsigned int id)
   brc = Bind("?ps", &retnick);
   if(Fetch(rc, brc) == 0)
   {
-    db_log("db_get_nickname_from_id: %d not found.\n", id);
+    db_log("db_get_nickname_from_id: %d not found.", id);
     Free(brc);
     Free(rc);
     return NULL;
@@ -497,7 +545,7 @@ db_get_nickname_from_nickid(unsigned int id)
   brc = Bind("?ps", &retnick);
   if(Fetch(rc, brc) == 0)
   {
-    db_log("db_get_nickname_from_nickid: %d not found.\n", id);
+    db_log("db_get_nickname_from_nickid: %d not found.", id);
     Free(brc);
     Free(rc);
     return NULL;
@@ -525,7 +573,7 @@ db_get_id_from_name(const char *name, unsigned int type)
   brc = Bind("?d", &ret);
   if(Fetch(rc, brc) == 0)
   {
-    db_log("db_get_id_from_name: '%s' not found.\n", name);
+    db_log("db_get_id_from_name: '%s' not found.", name);
     Free(brc);
     Free(rc);
     return 0;
@@ -651,7 +699,7 @@ db_forbid_nick(const char *n)
 
   if((nick = db_find_nick(n)) != NULL)
   {
-    db_delete_nick(nick->id, nick->nickid, nick->pri_nickid);
+    db_delete_nick(nick->id, nick->pri_nickid, nick->nickid);
     free_nick(nick);
   }
 
@@ -800,7 +848,9 @@ db_delete_nick(unsigned int accid, unsigned int nickid, unsigned int priid)
 
   if(TransCommit() != 0)
     return FALSE;
- 
+
+  execute_callback(on_nick_drop_cb, accid, nickid, priid);
+
   return TRUE;
 }
 
@@ -849,6 +899,7 @@ db_list_add(unsigned int type, const void *value)
   struct AccessEntry *aeval = (struct AccessEntry *)value;
   struct ChanAccess *caval  = (struct ChanAccess *)value;
   struct ServiceBan *banval = (struct ServiceBan *)value;
+  struct JupeEntry *jval    = (struct JupeEntry *)value;
   unsigned int id;
   int ret = 0;
 
@@ -857,6 +908,9 @@ db_list_add(unsigned int type, const void *value)
     case ACCESS_LIST:
       db_exec(ret, INSERT_NICKACCESS, aeval->id, aeval->value);
       break;
+    case CERT_LIST:
+      db_exec(ret, INSERT_NICKCERT, aeval->id, aeval->value);
+      break;
     case AKILL_LIST:
       db_exec(ret, INSERT_AKILL, banval->mask, banval->reason, 
           banval->setter, banval->time_set, banval->duration);
@@ -866,10 +920,9 @@ db_list_add(unsigned int type, const void *value)
           banval->time_set, banval->duration);
       break;
     case AKICK_LIST:
-      id = db_get_id_from_name(banval->channel, GET_CHANID_FROM_CHAN);
       if(banval->target != 0)
       {
-        db_exec(ret, INSERT_AKICK_ACCOUNT, id, banval->target,
+        db_exec(ret, INSERT_AKICK_ACCOUNT, banval->channel, banval->target,
             banval->setter, banval->reason, banval->time_set, 
             banval->duration);
       }
@@ -886,6 +939,9 @@ db_list_add(unsigned int type, const void *value)
       db_exec(ret, INSERT_CHANACCESS, caval->account, caval->channel,
           caval->level);
       break;
+    case JUPES_LIST:
+      db_exec(ret, INSERT_JUPE, jval->setter, jval->name, jval->reason);
+      break;
     default:
       assert(1 == 0);
       break;
@@ -907,7 +963,8 @@ db_list_first(unsigned int type, unsigned int param, void **entry)
   struct ServiceBan *banval;
   struct DBResult *result;
   struct InfoChanList *info;
-  unsigned int query, level;
+  struct JupeEntry *jval;
+  unsigned int query;
   
   switch(type)
   {
@@ -917,6 +974,18 @@ db_list_first(unsigned int type, unsigned int param, void **entry)
       *entry = aeval;
       brc = Bind("?d?ps", &aeval->id, &aeval->value);
       break;
+    case CERT_LIST:
+      query = GET_NICKCERT;
+      aeval = MyMalloc(sizeof(struct AccessEntry));
+      *entry = aeval;
+      brc = Bind("?d?ps", &aeval->id, &aeval->value);
+      break;
+    case JUPES_LIST:
+      query = GET_JUPES;
+      jval = MyMalloc(sizeof(struct JupeEntry));
+      *entry = jval;
+      brc = Bind("?d?ps?ps?d", &jval->id, &jval->name, &jval->reason, &jval->setter);
+      break;
     case ADMIN_LIST:
       query = GET_ADMINS;
       *entry = strval;
@@ -940,7 +1009,7 @@ db_list_first(unsigned int type, unsigned int param, void **entry)
 
       banval = MyMalloc(sizeof(struct ServiceBan));
       *entry = banval;
-      brc = Bind("?d?ps?d?d?ps?ps?d?d", &banval->id, &banval->channel,
+      brc = Bind("?d?d?d?d?ps?ps?d?d", &banval->id, &banval->channel,
           &banval->target, &banval->setter, &banval->mask, 
           &banval->reason, &banval->time_set, &banval->duration);
       break;
@@ -957,7 +1026,7 @@ db_list_first(unsigned int type, unsigned int param, void **entry)
 
       info = MyMalloc(sizeof(struct InfoChanList));
       *entry = info;
-      brc = Bind("?ps?d", &info->channel, &level);
+      brc = Bind("?d?ps?d", &info->channel_id, &info->channel, &info->ilevel);
       break;
     case CHMASTER_LIST:
       query = GET_CHAN_MASTERS;
@@ -1035,12 +1104,16 @@ db_list_first(unsigned int type, unsigned int param, void **entry)
   {
     DupString(banval->mask, banval->mask);
     DupString(banval->reason, banval->reason);
-    DupString(banval->channel, banval->channel);
     banval->type = AKICK_BAN;
   }
+  else if(type == JUPES_LIST)
+  {
+    DupString(jval->name, jval->name);
+    DupString(jval->reason, jval->reason);
+  }
   else if(type == NICKCHAN_LIST)
   {
-    switch(level)
+    switch(info->ilevel)
     {
       case MASTER_FLAG:
         info->level = "MASTER";
@@ -1065,17 +1138,24 @@ db_list_next(void *result, unsigned int type, void **entry)
   struct ChanAccess *caval;
   struct ServiceBan *banval;
   struct InfoChanList *info;
+  struct JupeEntry *jval;
   char *strval = (char*)*entry; 
-  unsigned int level;
- 
+
   switch(type)
   {
     case ACCESS_LIST:
+    case CERT_LIST:
       aeval = MyMalloc(sizeof(struct AccessEntry));
       *entry = aeval;
       Free(res->brc);
       res->brc = Bind("?d?ps", &aeval->id, &aeval->value);
       break;
+    case JUPES_LIST:
+      jval = MyMalloc(sizeof(struct JupeEntry));
+      *entry = jval;
+      Free(res->brc);
+      res->brc = Bind("?d?ps?ps?d", &jval->id, &jval->name, &jval->reason, &jval->setter);
+      break;
     case ADMIN_LIST:
     case NICKLINK_LIST:
     case CHMASTER_LIST:
@@ -1115,7 +1195,7 @@ db_list_next(void *result, unsigned int type, void **entry)
       info = MyMalloc(sizeof(struct InfoChanList));
       *entry = info;
       Free(res->brc);
-      res->brc = Bind("?ps?d", &info->channel, &level);
+      res->brc = Bind("?d?ps?d", &info->channel_id, &info->channel, &info->ilevel);
       break;
     default:
       assert(0 == 1);
@@ -1135,11 +1215,15 @@ db_list_next(void *result, unsigned int type, void **entry)
     banval->type = AKICK_BAN;
     DupString(banval->mask, banval->mask);
     DupString(banval->reason, banval->reason);
-    DupString(banval->channel, banval->channel);
+  }
+  else if(type == JUPES_LIST)
+  {
+    DupString(jval->name, jval->name);
+    DupString(jval->reason, jval->reason);
   }
   else if(type == NICKCHAN_LIST)
   {
-    switch(level)
+    switch(info->ilevel)
     {
       case MASTER_FLAG:
         info->level = "MASTER";
@@ -1294,7 +1378,7 @@ db_find_chan(const char *channel)
 
   if(Fetch(rc, brc) == 0)
   {
-    db_log("db_find_chan: '%s' not found.\n", channel);
+    db_log("db_find_chan: '%s' not found.", channel);
     Free(brc);
     Free(rc);
     free_regchan(channel_p);
@@ -1310,7 +1394,7 @@ db_find_chan(const char *channel)
   DupString(channel_p->topic, channel_p->topic);
   DupString(channel_p->mlock, channel_p->mlock);
 
-  db_log("db_find_chan: Found chan %s\n", channel_p->channel);
+  db_log("db_find_chan: Found chan %s", channel_p->channel);
 
   Free(brc);
   Free(rc);
@@ -1368,6 +1452,9 @@ db_delete_chan(const char *chan)
 
   if(ret == -1)
     return FALSE;
+
+  execute_callback(on_chan_drop_cb, chan);
+
   return TRUE;
 }
 
@@ -1410,24 +1497,25 @@ db_find_akill(const char *mask)
 
   if(rc == NULL)
     return NULL;
- 
+
   akill = MyMalloc(sizeof(struct ServiceBan));
- 
+
   brc = Bind("?d?ps?ps?d?d?d", &akill->id, &akill->mask, &akill->reason,
       &akill->setter, &akill->time_set, &akill->duration);
 
   if(Fetch(rc, brc) == 0)
   {
-    db_log("db_find_akill: '%s' not found.\n", mask);
+    db_log("db_find_akill: '%s' not found.", mask);
     Free(brc);
     Free(rc);
+    free_serviceban(akill);
     return NULL;
   }
 
   DupString(akill->mask, akill->mask);
   DupString(akill->reason, akill->reason);
 
-  db_log("db_find_akill: Found akill %s(asked for %s)\n", akill->mask, mask);
+  db_log("db_find_akill: Found akill %s(asked for %s)", akill->mask, mask);
 
   Free(brc);
   Free(rc);
@@ -1435,13 +1523,72 @@ db_find_akill(const char *mask)
   return akill;
 }
 
+struct JupeEntry *
+db_find_jupe(const char *name)
+{
+  yada_rc_t *rc, *brc;
+  struct JupeEntry *jupe;
+
+  assert(name != NULL);
+
+  db_query(rc, FIND_JUPE, name);
+
+  if(rc == NULL)
+    return NULL;
+
+  jupe = MyMalloc(sizeof(struct JupeEntry));
+
+  brc = Bind("?d?ps?ps?d", &jupe->id, &jupe->name, &jupe->reason, &jupe->setter);
+
+  if(Fetch(rc, brc) == 0)
+  {
+    db_log("db_find_jupe: '%s' not found.", name);
+    Free(brc);
+    Free(rc);
+    free_jupeentry(jupe);
+    return NULL;
+  }
+
+  DupString(jupe->name, jupe->name);
+  DupString(jupe->reason, jupe->reason);
+
+  db_log("db_find_jupe: Found jupe %s [%s]", jupe->name, jupe->reason);
+
+  Free(brc);
+  Free(rc);
+
+  return jupe;
+}
+
 int
 db_get_num_masters(unsigned int chanid)
 {
   yada_rc_t *rc, *brc;
   int count;
 
-  db_query(rc, GET_CHAN_MASTER_COUNT, chanid, MASTER_FLAG);
+  db_query(rc, GET_CHAN_MASTER_COUNT, chanid);
+
+  if(rc == NULL)
+    return 0;
+
+  brc = Bind("?d", &count);
+
+  if(Fetch(rc, brc) == 0)
+    count = 0;
+
+  Free(brc);
+  Free(rc);
+
+  return count;
+}
+
+int
+db_get_num_channel_accesslist_entries(unsigned int chanid)
+{
+  yada_rc_t *rc, *brc;
+  int count;
+
+  db_query(rc, COUNT_CHANNEL_ACCESS_LIST, chanid);
 
   if(rc == NULL)
     return 0;
