@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: interface.c 1049 2007-05-23 16:27:48Z swalsh $
+ *  $Id: interface.c 1182 2007-10-26 17:07:49Z swalsh $
  */
 
 #include "stdinc.h"
@@ -46,6 +46,9 @@ struct Callback *send_newserver_cb;
 struct Callback *send_join_cb;
 struct Callback *send_part_cb;
 struct Callback *send_nosuchsrv_cb;
+struct Callback *send_chops_notice_cb;
+struct Callback *send_squit_cb;
+
 static BlockHeap *services_heap  = NULL;
 
 struct Callback *on_nick_change_cb;
@@ -63,6 +66,11 @@ struct Callback *on_topic_change_cb;
 struct Callback *on_privmsg_cb;
 struct Callback *on_notice_cb;
 struct Callback *on_burst_done_cb;
+struct Callback *on_certfp_cb;
+struct Callback *on_db_init_cb;
+
+struct Callback *on_nick_drop_cb;
+struct Callback *on_chan_drop_cb;
 
 struct LanguageFile ServicesLanguages[LANG_LAST];
 struct ModeList *ServerModeList;
@@ -92,12 +100,15 @@ init_interface()
   send_join_cb        = register_callback("Send JOIN", NULL);
   send_part_cb        = register_callback("Send PART", NULL);
   send_nosuchsrv_cb   = register_callback("Send No such server", NULL);
+  send_chops_notice_cb = register_callback("Send NOTICE to channel ops", NULL);
+  send_squit_cb       = register_callback("Send SQUIT Message", NULL);
   on_nick_change_cb   = register_callback("Propagate NICK", NULL);
   on_join_cb          = register_callback("Propagate JOIN", NULL);
   on_part_cb          = register_callback("Propagate PART", NULL);
   on_quit_cb          = register_callback("Propagate QUIT", NULL);
   on_umode_change_cb  = register_callback("Propagate UMODE", NULL);
   on_cmode_change_cb  = register_callback("Propagate CMODE", NULL);
+  on_squit_cb         = register_callback("Propagate SQUIT", NULL);
   on_identify_cb      = register_callback("Identify Callback", NULL);
   on_newuser_cb       = register_callback("New user coming to us", NULL);
   on_channel_created_cb = register_callback("Channel is being created", NULL);
@@ -107,6 +118,10 @@ init_interface()
   on_notice_cb        = register_callback("Notice for channel received", NULL);
   on_burst_done_cb    = register_callback("Notification that burst is complete", 
       NULL);
+  on_certfp_cb        = register_callback("Client certificate recieved for this user", NULL);
+  on_nick_drop_cb     = register_callback("Nick Dropped", NULL);
+  on_chan_drop_cb     = register_callback("Chan Dropped", NULL);
+  on_db_init_cb       = register_callback("On Database Init", NULL);
 
   load_language(ServicesLanguages, "services.en");
 }
@@ -135,6 +150,8 @@ cleanup_interface()
   unregister_callback(send_join_cb);
   unregister_callback(send_part_cb);
   unregister_callback(send_nosuchsrv_cb);
+  unregister_callback(send_chops_notice_cb);
+  unregister_callback(send_squit_cb);
   unregister_callback(on_nick_change_cb);
   unregister_callback(on_join_cb);
   unregister_callback(on_part_cb);
@@ -149,6 +166,9 @@ cleanup_interface()
   unregister_callback(on_privmsg_cb);
   unregister_callback(on_notice_cb);
   unregister_callback(on_burst_done_cb);
+  unregister_callback(on_certfp_cb);
+  unregister_callback(on_nick_drop_cb);
+  unregister_callback(on_chan_drop_cb);
 
   unload_languages(ServicesLanguages);
 }
@@ -166,9 +186,10 @@ make_service(char *name)
 }
 
 struct Client *
-introduce_client(const char *name)
+introduce_client(const char *name, const char *gecos, char isservice)
 {
-  struct Client *client; 
+  struct Client *client;
+  char *umode;
   dlink_node *ptr;
 
 
@@ -183,14 +204,19 @@ introduce_client(const char *name)
     strlcpy(client->name, name, sizeof(client->name));
     hash_add_client(client);
 
-    register_remote_user(&me, client, "services", me.name, me.name, name);
+    register_remote_user(&me, client, "services", me.name, me.name, gecos);
   }
 
   /* If we are not connected yet, the service will be sent as part of burst */
   if(me.uplink != NULL)
   {
+    if(isservice)
+      umode = "oP";
+    else
+      umode = "P";
+
     execute_callback(send_newuser_cb, me.uplink, name, "services", me.name,
-      name, "");
+      gecos, umode);
     DLINK_FOREACH(ptr, client->channel.head)
     {
       struct Membership *ms = ptr->data;
@@ -204,28 +230,37 @@ introduce_client(const char *name)
 struct Client*
 introduce_server(const char *name, const char *gecos)
 {
-  struct Client *client;
-    
-  if((client = find_client(name)) != NULL)
-    return client;
-  
-  client = make_client(&me);
+  struct Client *client = find_server(name);
+  ilog(L_DEBUG, "Introducing Server %s [%s]", name, gecos);
 
-  client->status |= STAT_SERVER;
+  if(client == NULL)
+  {
+    ilog(L_DEBUG, "Server %s not found", name);
+    client = make_client(&me);
+    SetServer(client);
 
-  client->servptr = &me;
-  dlinkAdd(client, &client->lnode, &client->servptr->server_list);
+    client->servptr = &me;
+    dlinkAdd(client, &client->lnode, &client->servptr->server_list);
 
-  client->tsinfo = CurrentTime;
-  //I don't think this is needed because it's on the server list
-  //dlinkAdd(client, &client->node, &global_client_list);
+    client->tsinfo = CurrentTime;
+
+    strlcpy(client->name, name, sizeof(client->name));
+    strlcpy(client->info, gecos, sizeof(client->info));
+    hash_add_client(client);
+
+    dlinkAdd(client, &client->node, &global_client_list);
+  }
 
-  strlcpy(client->name, name, sizeof(client->name));
-  strlcpy(client->info, gecos, sizeof(client->info));
-  hash_add_client(client);
 
   if(me.uplink != NULL)
+  {
+    ilog(L_DEBUG, "Sending New Server %s [%s]", client->name, client->info);
     execute_callback(send_newserver_cb, client);
+  }
+  else
+  {
+    ilog(L_DEBUG, "Waiting to introduce %s until connected", client->name);
+  }
 
   return client;
 }
@@ -246,6 +281,17 @@ join_channel(struct Client *service, struct Channel *channel)
   return channel;
 }
 
+void
+squit_server(const char *server, const char *reason)
+{
+  struct Client *serv = find_server(server);
+  if(serv != NULL)
+  {
+    execute_callback(send_squit_cb, &me, server, reason);
+    exit_client(serv, &me, reason);
+  }
+}
+
 void
 part_channel(struct Client *service, const char *chname, const char *reason)
 {
@@ -390,6 +436,7 @@ reply_mail(struct Service *service, struct Client *client,
   fprintf(ptr, "To: %s\n", client->nickname->email);
   fprintf(ptr, "From: %s\n", Mail.from_address);
   fprintf(ptr, "Subject: %s\n", subjectstr);
+  fprintf(ptr, "Precedence: junk\n");
 
   bufptr = buf;
   while(*bufptr != '\0')
@@ -425,12 +472,8 @@ kill_user(struct Service *service, struct Client *client, const char *reason)
     ilog(L_DEBUG, "Was going to kill: %s (%s)", client->name, reason);
   else
   {
-    if(!IsKilled(client))
-    {
-      execute_callback(send_kill_cb, me.uplink, service, client, reason);
-      exit_client(client, &me, reason);
-      SetKilled(client);
-    }
+    execute_callback(send_kill_cb, me.uplink, service, client, reason);
+    exit_client(client, &me, reason);
   }
 }
 
@@ -470,14 +513,32 @@ akill_add(struct Service *service, struct Client *client, const char* mask,
     return NULL;
   }
 
-  ilog(L_NOTICE, "%s Added an akill on %s. Expires %d [%s]", client->name,
-      akill->mask, duration, reason);
+  ilog(L_NOTICE, "%s Added an akill on %s. Expires %s [%s]", client->name,
+      akill->mask, smalldate(akill->time_set + duration), reason);
 
   send_akill(service, client->name, akill);
 
   return akill;
 }
 
+void
+send_chops_notice(struct Service *service, struct Channel *chptr, 
+    const char *format, ...)
+{
+  va_list ap;
+  char *buf;
+
+  if(chptr == NULL || chptr->regchan == NULL || !chptr->regchan->verbose)
+    return;
+
+  va_start(ap, format);
+  vasprintf(&buf, format, ap);
+  va_end(ap);
+
+  execute_callback(send_chops_notice_cb, me.uplink, service, chptr, buf);
+  MyFree(buf);
+}
+
 void
 send_umode(struct Service *service, struct Client *client, const char *mode)
 {
@@ -561,11 +622,23 @@ void
 kick_user(struct Service *service, struct Channel *chptr, const char *client, 
     const char *reason)
 {
+  struct Membership *ms;
+  struct Client *ptr = find_client(client);
+
   if(ServicesState.debugmode)
     return;
 
+  if((ms = find_channel_link(ptr, chptr)) == NULL)
+  {
+    ilog(L_CRIT, "Tried to remove %s from channel %s they werent on",
+        ptr->name, chptr->chname);
+    return;
+  }
+
   execute_callback(send_kick_cb, me.uplink, service->name, chptr->chname, 
       client, reason);
+
+  remove_user_from_channel(ms);
 }
 
 static void 
@@ -690,6 +763,16 @@ quiet_mask(struct Service *service, struct Channel *chptr, const char *mask)
   add_id(client, chptr, (char *)mask, CHFL_QUIET);
 }
 
+void
+unquiet_mask(struct Service *service, struct Channel *chptr, const char *mask)
+{
+  if(ServicesState.debugmode)
+    return;
+
+  send_cmode(service, chptr, "-q", mask);
+  del_id(chptr, (char*)mask, CHFL_QUIET);
+}
+
 void
 identify_user(struct Client *client)
 {
@@ -856,7 +939,8 @@ check_list_entry(unsigned int type, unsigned int id, const char *value)
   return FALSE;
 }
 
-int enforce_matching_serviceban(struct Service *service, struct Channel *chptr, 
+int 
+enforce_matching_serviceban(struct Service *service, struct Channel *chptr, 
     struct Client *client)
 {
   struct ServiceBan *sban;
@@ -977,7 +1061,10 @@ enforce_client_serviceban(struct Service *service, struct Channel *chptr,
     {
       if(sban->type == AKICK_BAN)
       {
-        ban_mask(service, chptr, sban->mask);
+        char banmask[IRC_BUFSIZE+1];
+
+        ircsprintf(banmask, "%s", sban->mask);
+        ban_mask(service, chptr, banmask);
         kick_user(service, chptr, client->name, sban->reason);
       }
       else if(sban->type == AKILL_BAN)
@@ -1129,6 +1216,8 @@ set_mode_lock(struct Service *service, const char *channel,
             return FALSE;
           }
         }
+        else
+          l = -1;
         break;
       case 'k':
         if(dir)
@@ -1141,6 +1230,8 @@ set_mode_lock(struct Service *service, const char *channel,
             }
             strlcpy(key, parv[para++], sizeof(key));
           }
+        else
+          k = -1;
         break;
     }
   }
@@ -1156,12 +1247,20 @@ set_mode_lock(struct Service *service, const char *channel,
   get_modestring(setmodes, setstr, MODEBUFLEN/2);
   get_modestring(delmodes, delstr, MODEBUFLEN/2);
 
+  if(l < 0)
+    strlcat(delstr, "l", MODEBUFLEN/2);
+
+  if(k < 0)
+    strlcat(delstr, "k", MODEBUFLEN/2);
+
+  ilog(L_DEBUG, "MLOCK -%s+%s", delstr, setstr);
+
   /* If we've been asked to update the db, then we should do so. */
   if(value != NULL)
   {
     char *lk = "";
 
-    k = l = s = d = FALSE;
+    s = d = FALSE;
 
     if(*setstr != '\0')
       s = TRUE;
@@ -1190,12 +1289,15 @@ set_mode_lock(struct Service *service, const char *channel,
     }
 
     snprintf(mlockbuf, MODEBUFLEN, "%s%s%s%s%s%s",
-        s || l || k ? "+": "", 
+        s || l > 0 || k > 0 ? "+": "", 
         s ? setstr : "", 
         lk,
         d ? "-" : "",
         d ? delstr : "",
-        l || k ? parabuf : "");
+        l > 0 || k > 0? parabuf : "");
+
+    if(regchptr->autolimit && l < 0)
+      regchptr->autolimit = FALSE;
 
     if(!db_set_string(SET_CHAN_MLOCK, regchptr->id, 
           *mlockbuf == '\0' ? NULL : mlockbuf))
@@ -1235,6 +1337,20 @@ set_mode_lock(struct Service *service, const char *channel,
   c = delstr;
   while(*c != '\0')
   {
+    if(*c == 'l')
+    {
+      chptr->mode.limit = 0;
+      c++;
+      continue;
+    }
+
+    if(*c == 'k')
+    {
+      chptr->mode.key[0] = '\0';
+      c++;
+      continue;
+    }
+
     mode = get_mode_from_letter(*c);
     if(mode <= 0)
     {
@@ -1250,12 +1366,18 @@ set_mode_lock(struct Service *service, const char *channel,
   get_modestring(delmodes, delstr, MODEBUFLEN/2);
   chptr->mode.mode |= setmodes;
   chptr->mode.mode &= ~delmodes;
-  if(l)
+
+  if(l > 0)
     chptr->mode.limit = limit;
-  if(k)
+  else if (l < 0)
+    strlcat(delstr, "l", MODEBUFLEN/2);
+
+  if(k > 0)
     strcpy(chptr->mode.key, key);
- 
-  /* 
+  else if (l < 0)
+    strlcat(delstr, "k", MODEBUFLEN/2);
+
+  /*
    * Set up the modestring and parameter(s) and set them. This could probably
    * be written a little better.
    */
@@ -1327,10 +1449,20 @@ free_serviceban(struct ServiceBan *ban)
   ilog(L_DEBUG, "Freeing serviceban %p for %s", ban, ban->mask);
   MyFree(ban->mask);
   MyFree(ban->reason);
-  MyFree(ban->channel);
   MyFree(ban);
 }
 
+void
+free_jupeentry(struct JupeEntry *entry)
+{
+	ilog(L_DEBUG, "Freeing JupeEntry %p for %s", entry, entry->name);
+	MyFree(entry->name);
+	MyFree(entry->reason);
+  entry->name = NULL;
+  entry->reason = NULL;
+	MyFree(entry);
+}
+
 int 
 check_nick_pass(struct Nick *nick, const char *password)
 {
@@ -1440,3 +1572,67 @@ valid_wild_card(const char *arg)
 
   return 0;
 }
+
+char *
+check_masterless_channels(unsigned int accid)
+{
+  struct InfoChanList *chan;
+  void *listptr, *first;
+
+  first = listptr = NULL;
+
+  if((listptr = db_list_first(NICKCHAN_LIST, accid, (void**)&chan)) != NULL)
+  {
+    first = listptr;
+
+    while(listptr != NULL)
+    {
+      if(db_get_num_channel_accesslist_entries(chan->channel_id) == 1)
+      {
+        char *nick = db_get_nickname_from_id(accid);
+        struct Channel *chptr;
+
+        ilog(L_NOTICE, "Dropping channel %s because its access list would be "
+            "left empty by drop of nickname %s", chan->channel, nick);
+        MyFree(nick);
+        db_delete_chan(chan->channel);
+
+        chptr = hash_find_channel(chan->channel);
+        if(chptr != NULL)
+        {
+          if(chptr->regchan != NULL)
+          {
+            free_regchan(chptr->regchan);
+            chptr->regchan = NULL;
+          }
+        }
+
+        MyFree(chan);
+        listptr = db_list_next(listptr, NICKCHAN_LIST, (void**)&chan);
+        continue;
+      }
+      if(chan->ilevel == MASTER_FLAG && db_get_num_masters(chan->channel_id) <= 1)
+      {
+        char *cname;
+        
+        DupString(cname, chan->channel);
+        db_list_done(listptr);
+
+        MyFree(chan);
+        return cname;
+      }
+      MyFree(chan);
+      listptr = db_list_next(listptr, NICKCHAN_LIST, (void**)&chan);
+    }
+    MyFree(chan);
+    db_list_done(first);
+  }
+  else
+  {
+    MyFree(chan);
+    if(listptr != NULL)
+      db_list_done(listptr);
+  }
+
+  return 0;
+}
