@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: parse.c 1049 2007-05-23 16:27:48Z swalsh $
+ *  $Id: parse.c 1051 2007-05-24 14:29:13Z swalsh $
  */
 
 #include "stdinc.h"
@@ -787,7 +787,7 @@ recurse_help_messages(struct Service *service, struct Client *client,
 
   if (mtree->msg != NULL && !(mtree->msg->flags & SFLG_ALIAS))
   {
-    if(mtree->msg->flags & SFLG_CHANARG || mtree->msg->access < OPER_FLAG ||
+    if(mtree->msg->flags & SFLG_CHANARG || 
         client->access >= mtree->msg->access)
       reply_user(service, service, client, mtree->msg->help_short, 
           mtree->msg->cmd);
