@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: services.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: services.c 1101 2007-06-28 20:32:59Z tjfontaine $
  */
 
 #include "stdinc.h"
@@ -48,7 +48,9 @@ static struct lgetopt myopts[] = {
   {"version",    &ServicesState.printversion,
    YESNO, "Print version and exit"},
   {"debugmode",  &ServicesState.debugmode,
-      YESNO, "Set debug mode and disable all interaction"},
+   YESNO, "Set debug mode and disable all interaction"},
+  {"keepmodules", &ServicesState.keepmodules,
+   YESNO, "Stops modules from being unloaded.  Helps debugging memory leaks"},
   {"help", NULL, USAGE, "Print this text"},
   {NULL, NULL, STRING, NULL},
 };
@@ -214,6 +216,7 @@ int main(int argc, char *argv[])
   init_client();
   init_parser();
   init_channel_modes();
+  init_mqueue();
 
   me.from = me.servptr = &me;
   SetServer(&me);
@@ -322,6 +325,7 @@ services_die(const char *msg, int rboot)
   cleanup_client();
   cleanup_channel();
   cleanup_interface();
+  cleanup_mqueue();
   unregister_callback(iorecv_cb);
   unregister_callback(connected_cb);
   unregister_callback(iosend_cb);
